/****************************************************************************************************************
   Program: 020_npi.sas

   Purpose: Read in NPPES data from April 2024, FY2023 and CY2021 to identify the primary taxonomy and practice location 
			in each period for all individual NPIs nationwide. Merge with the taxonomy crosswalk file from the 010 program 
			to identify NPIs with the relevant licensures and specialties for wave 1 of the Community Clinician sample frame.

   Input files: NPPES_JAN2021.sas7bdat - NPPES_DEC2021.sas7bdat
                NPPES_OCT2022.sas7bdat - NPPES_SEPT2023.sas7bdat
                NPPES_APR2024.sas7bdat
                _010_taxonomy_xwalk.sas7bdat
                npidata_pfile_20050523-20240407.csv

   Output files: _020_npi.sas7bdat
                 _020_npi.csv
****************************************************************************************************************/
options nocenter mprint obs=max;

%let input_data = <input directory>;
%let csvout = <output directory>;

libname lookups "<input directory>" access = readonly;
libname out "<output directory>";

%let taxonomy_xwalk = lookups._010_taxonomy_xwalk;
%let outdata = out._020_npi;
%let csvoutdata = &csvout./_020_npi.csv;

proc format;
	value spclty
		1 = "PCP"
		3 = "Psych"
		6 = "Neurology"
	;
run;

/* Read in the taxonomy crosswalk from the 010 program and limit to unique taxonomy codes related to primary care, neurology, and psychology */
proc sort data = &taxonomy_xwalk. (where = (spclty_updt_ndws in (1, 3, 6))) out = taxonomy_include (keep = prvdr_txnmy_cd licensing spclty_updt_ndws) nodupkey;
	by prvdr_txnmy_cd licensing spclty_updt_ndws;
run;

data taxonomy_include;
	set taxonomy_include;
	format spclty_updt_ndws spclty.;
	rename spclty_updt_ndws = dm_specialty;
run;

/* Import the April 2024 NPPES file (because the current year's NPPES files were not yet available on the VRDC) */
/* Ignore tmp[x] variables - they are not needed and will be dropped */
data nppes_apr2024;
	infile "&input_data./npidata_pfile_20050523-20240407.csv"
	delimiter = ","
	missover
	firstobs = 2
	dsd
	lrecl = 32767;
	
	format npi $10.;
    format entity_type_code $1.;
	format tmp1 $1.;
	format tmp2 $1.;
	format tmp3 $1.;
    format prov_ln $35.;
    format prov_fn $20.;
	format tmp4 $1.;
    format prov_nm_pre $5.;
    format prov_nm_suff $5.;
	format tmp5 $1.;
	format tmp6 $1.;
	format tmp7 $1.;
	format tmp8 $1.;
	format tmp9 $1.;
	format tmp10 $1.;
	format tmp11 $1.;
	format tmp12 $1.;
	format tmp13 $1.;
	format tmp14 $1.;
    format prov_mail1 $55.;
    format prov_mail2 $55.;
    format prov_city $40.;
    format prov_state $40.;
    format prov_zip $20.;
    format prov_country $2.;
    format prov_phone $20.;
	format tmp15 $1.;
    format prov_loc1 $55.;
    format prov_loc2 $55.;
    format prov_loc_city $40.;
    format prov_loc_state $40.;
    format prov_loc_zip $20.;
    format prov_loc_country $2.;
    format prov_loc_phone $20.;
	format tmp16 $1.;
    format enum_date_tmp $10.;
	format tmp17 $1.;
    format deact_reason $2.;
    format deact_date_tmp $10.;
    format react_date_tmp $10.;
	format tmp18 $1.;
	format tmp19 $1.;
	format tmp20 $1.;
	format tmp21 $1.;
	format tmp22 $1.;
	format tmp23 $1.;
    format hptc_1 $10.;
	format tmp24 $1.;
	format tmp25 $1.;
    format hppts_1 $1.;
    format hptc_2 $10.;
	format tmp26 $1.;
	format tmp27 $1.;
    format hppts_2 $1.;
    format hptc_3 $10.;
	format tmp28 $1.;
	format tmp29 $1.;
    format hppts_3 $1.;
    format hptc_4 $10.;
	format tmp30 $1.;
	format tmp31 $1.;
    format hppts_4 $1.;
    format hptc_5 $10.;
	format tmp32 $1.;
	format tmp33 $1.;
    format hppts_5 $1.;
    format hptc_6 $10.;
	format tmp34 $1.;
	format tmp35 $1.;
    format hppts_6 $1.;
    format hptc_7 $10.;
	format tmp36 $1.;
	format tmp37 $1.;
    format hppts_7 $1.;
    format hptc_8 $10.;
	format tmp38 $1.;
	format tmp39 $1.;
    format hppts_8 $1.;
    format hptc_9 $10.;
	format tmp40 $1.;
	format tmp41 $1.;
    format hppts_9 $1.;
    format hptc_10 $10.;
	format tmp42 $1.;
	format tmp43 $1.;
    format hppts_10 $1.;
    format hptc_11 $10.;
	format tmp44 $1.;
	format tmp45 $1.;
    format hppts_11 $1.;
    format hptc_12 $10.;
	format tmp46 $1.;
	format tmp47 $1.;
    format hppts_12 $1.;
    format hptc_13 $10.;
	format tmp48 $1.;
	format tmp49 $1.;
    format hppts_13 $1.;
    format hptc_14 $10.;
	format tmp50 $1.;
	format tmp51 $1.;
    format hppts_14 $1.;
    format hptc_15 $10.;
	format tmp52 $1.;
	format tmp53 $1.;
    format hppts_15 $1.;

	input npi $      
          entity_type_code $      
	      tmp1 $      
	      tmp2 $      
	      tmp3 $      
          prov_ln $
          prov_fn $
	      tmp4 $      
          prov_nm_pre $
          prov_nm_suff $
	      tmp5 $      
	      tmp6 $      
	      tmp7 $      
	      tmp8 $      
	      tmp9 $      
	      tmp10 $      
	      tmp11 $      
	      tmp12 $      
	      tmp13 $      
	      tmp14 $      
          prov_mail1 $
          prov_mail2 $
          prov_city $
          prov_state $
          prov_zip $
          prov_country $
          prov_phone $
	      tmp15 $      
          prov_loc1 $
          prov_loc2 $
          prov_loc_city $
          prov_loc_state $
          prov_loc_zip $
          prov_loc_country $
          prov_loc_phone $
	      tmp16 $      
          enum_date_tmp $      
	      tmp17 $  
          deact_reason $ 
          deact_date_tmp $      
          react_date_tmp $      
	      tmp18 $      
	      tmp19 $      
	      tmp20 $      
	      tmp21 $      
	      tmp22 $      
	      tmp23 $      
          hptc_1 $      
	      tmp24 $      
	      tmp25 $      
          hppts_1 $      
          hptc_2 $      
	      tmp26 $      
	      tmp27 $      
          hppts_2 $      
          hptc_3 $      
	      tmp28 $      
	      tmp29 $      
          hppts_3 $      
          hptc_4 $      
	      tmp30 $      
	      tmp31 $      
          hppts_4 $      
          hptc_5 $      
	      tmp32 $      
	      tmp33 $      
          hppts_5 $      
          hptc_6 $      
	      tmp34 $      
	      tmp35 $      
          hppts_6 $      
          hptc_7 $      
	      tmp36 $      
	      tmp37 $      
          hppts_7 $      
          hptc_8 $      
	      tmp38 $      
	      tmp39 $      
          hppts_8 $      
          hptc_9 $      
	      tmp40 $      
	      tmp41 $      
          hppts_9 $      
          hptc_10 $      
	      tmp42 $      
	      tmp43 $      
          hppts_10 $      
          hptc_11 $      
	      tmp44 $      
	      tmp45 $      
          hppts_11 $      
          hptc_12 $      
	      tmp46 $      
	      tmp47 $      
          hppts_12 $      
          hptc_13 $      
	      tmp48 $      
	      tmp49 $      
          hppts_13 $      
          hptc_14 $      
	      tmp50 $      
	      tmp51 $      
          hppts_14 $      
          hptc_15 $      
	      tmp52 $      
	      tmp53 $      
          hppts_15 $
	;
run;

data nppes_apr2024;
	set nppes_apr2024 (where=(entity_type_code = "1"));
	deact_date = input(deact_date_tmp, mmddyy10.);
	react_date = input(react_date_tmp, mmddyy10.);
	enum_date = input(enum_date_tmp, mmddyy10.);
	format deact_date react_date enum_date mmddyy10.;
	drop tmp: deact_date_tmp react_date_tmp enum_date_tmp;
run;

/* Macro to read in and clean NPPES data */
%macro pull_process_nppes(nppes_files, yr);
	
	/* Read in each monthly NPPES file and limit to individual NPIs / select variables */
	%do i = 1 %to %sysfunc(countw(&nppes_files., " "));
		%let nppes_file = %scan(&nppes_files., &i., " ");
		data nppes_file_&yr._m&i.;
			set &nppes_file. (where = (entity_type_code = '1'));
			month_counter = &i.;
			keep npi 
				 prov_nm_pre 
				 prov_fn 
				 prov_ln 
				 prov_nm_suff 
				 enum_date 
				 deact_date 
				 deact_reason 
				 react_date
				 prov_loc1 
	 			 prov_loc2 
				 prov_loc_city
				 prov_loc_state 
				 prov_loc_zip 
				 prov_loc_country 
				 prov_loc_phone  
				 prov_mail1 
				 prov_mail2 
				 prov_city
				 prov_state 
				 prov_zip 
				 prov_country
				 prov_phone
				 hptc_1 - hptc_15
				 hppts_1 - hppts_15
				 month_counter
			;
			rename prov_nm_pre = prov_nm_pre_&yr.
				   prov_fn = prov_fn_&yr.
				   prov_ln = prov_ln_&yr.
				   prov_nm_suff = prov_nm_suff_&yr.
				   enum_date = enum_date_&yr.
				   deact_date = deact_date_&yr.
				   deact_reason = deact_reason_&yr.
				   react_date = react_date_&yr.
				   prov_loc1 = prctc_add_1_&yr.
	 			   prov_loc2 = prctc_add_2_&yr.
				   prov_loc_city = prctc_city_&yr.
				   prov_loc_state = prctc_state_&yr.
				   prov_loc_zip = prctc_zip_&yr.
				   prov_loc_country = prctc_cntry_&yr.
				   prov_loc_phone = prctc_phone_&yr.
				   prov_mail1 = mlg_add_1_&yr.
				   prov_mail2 = mlg_add_2_&yr.
				   prov_city = mlg_city_&yr.
				   prov_state = mlg_state_&yr.
				   prov_zip = mlg_zip_&yr.
				   prov_country = mlg_cntry_&yr.
				   prov_phone = mlg_phone_&yr.
			;
		run;
	%end;

	/* Stack the monthly NPPES files and determine the primary taxonomy */
	data nppes_all_months_&yr.;
		set nppes_file_&yr._m:;
		unknown_tax_counter_&yr. = 0;
		length taxonomy_&yr. $100.;
		taxonomy_&yr. = "";
		prim_tax_counter_&yr. = 0;
		no_tax_counter_&yr. = 0;
		%do j = 1 %to 15;
			if hppts_&j. = "X" then unknown_tax_counter_&yr.= unknown_tax_counter_&yr.+ 1;
			if hppts_&j. = "Y" then do;
				taxonomy_&yr. = hptc_&j.;
				prim_tax_counter_&yr. = prim_tax_counter_&yr. + 1;
			end;
			if missing(hptc_&j.) then no_tax_counter_&yr. = no_tax_counter_&yr. + 1;
		%end;
		missing_all_hptc_cds_&yr. = (no_tax_counter_&yr. = 15);
		year_&yr. = 1;
	run;

	/* Keep the last (most recent) entry for NPIs that are in multiple monthly datasets */
	proc sort data = nppes_all_months_&yr.;
		by npi month_counter;
	run;

	data nppes_selected_&yr.;
		set nppes_all_months_&yr.;
		by npi;
		if last.npi;
	run;

	proc transpose data = nppes_selected_&yr. (keep = npi taxonomy_&yr. hptc_: where = (missing(taxonomy_&yr.))) 
				   out = nppes_no_prim_tax_hptc_&yr. (rename = col1 = hptc rename =_NAME_ = hptc_position drop = taxonomy_&yr.);
		by npi taxonomy_&yr.;
		var hptc_1 - hptc_15;
	run;

	/* For NPIs without a primary taxonomy, do the following:
		1. Populate taxonomy_[yyyy] = the first hptc_[x] that is in the include taxonomy lookup.
		2. If no taxonomies in the include taxonomy lookup then use the first taxonomy code.
	*/
	proc sql;
		create table noprimtax_in_includetax_&yr. as
		select a.npi,
			   a.hptc_position,
			   a.hptc
		from nppes_no_prim_tax_hptc_&yr. as a
		inner join
		taxonomy_include as b
		on a.hptc = b.prvdr_txnmy_cd;
	quit;

	proc sort data = noprimtax_in_includetax_&yr.;
		by npi hptc_position;
	run;

	data noprimtax_in_includetax_&yr.;
		set noprimtax_in_includetax_&yr.;
		by npi;
		if first.npi;
		drop hptc_position;
		rename hptc = taxonomy_backfill_&yr.;
	run;

	proc sql;
		create table nppes_selected_fnltax_&yr. as
		select a.*,
			   b.taxonomy_backfill_&yr.
		from nppes_selected_&yr. as a
		left join
		noprimtax_in_includetax_&yr. as b
		on a.npi = b.npi;
	quit;

	data nppes_selected_fnltax_&yr.;
		set nppes_selected_fnltax_&yr. (rename = (taxonomy_&yr. = taxonomy_old_&yr. hptc_1 = hptc_1_&yr.));
		taxonomy_&yr. = taxonomy_old_&yr.;
		if missing(taxonomy_&yr.) and not missing(taxonomy_backfill_&yr.) then taxonomy_&yr. = taxonomy_backfill_&yr.;
		if missing(taxonomy_&yr.) and not missing(hptc_1_&yr.) then taxonomy_&yr. = hptc_1_&yr.;
	run;

	/* Create final yearly NPPES dataset */
	data nppes_final_&yr.;
		set nppes_selected_fnltax_&yr.;
		drop taxonomy_old_&yr. taxonomy_backfill_&yr. hptc: hppts: month_counter;
	run;

%mend pull_process_nppes;

/* Call the macro for 2021 */
%pull_process_nppes(%str(nppes21.nppes_jan2021
						 nppes21.nppes_feb2021
						 nppes21.nppes_mar2021
						 nppes21.nppes_apr2021
						 nppes21.nppes_may2021
						 nppes21.nppes_jun2021
						 nppes21.nppes_jul2021
						 nppes21.nppes_aug2021
						 nppes21.nppes_sep2021
						 nppes21.nppes_oct2021
						 nppes21.nppes_nov2021
						 nppes21.nppes_dec2021), 
					yr=2021);

/* Call the macro for 2023 */
%pull_process_nppes(%str(nppes22.nppes_oct2022
						 nppes22.nppes_nov2022
						 nppes22.nppes_dec2022
						 nppes23.nppes_jan2023
						 nppes23.nppes_feb2023
						 nppes23.nppes_mar2023
						 nppes23.nppes_apr2023
						 nppes23.nppes_may2023
						 nppes23.nppes_jun2023
						 nppes23.nppes_jul2023
						 nppes23.nppes_aug2023
						 nppes23.nppes_sep2023), 
					yr=2023);

/* Call the macro for 2024 */
%pull_process_nppes(%str(nppes_apr2024), 
					yr=2024);

/* Merge the 2021, 2023, and 2024 datasets so that the output file contains one observation per NPI; create 2024 active status */
data nppes_2021_2023_2024;
	merge nppes_final_2021 (in=a) nppes_final_2023 (in=b) nppes_final_2024 (in=c);
	by npi;
	in_2021 = a;
	in_2023 = b;
	in_2024 = c;

	* 2024 active status;
	check_active_stus_2024 = .;
	if year_2024 = 1 and not missing(deact_date_2024) and not missing(react_date_2024) then do;
		flag_deact_2024_gt_react_2024 = (deact_date_2024 > react_date_2024);
		if deact_date_2024 > react_date_2024 and deact_date_2024 <= mdy(4, 30, 2024) then check_active_stus_2024 = -1;
		else if deact_date_2024 <= react_date_2024 and react_date_2024 <= mdy(4, 30, 2024) then check_active_stus_2024 = 1;
	end;
	else if year_2024 = 1 and not missing(deact_date_2024) and missing(react_date_2024) then do;
		if deact_date_2024 <= mdy(4, 30, 2024) then check_active_stus_2024 = -2; 
	end;
	else if year_2024 = 1 and missing(deact_date_2024) and not missing(deact_reason_2024) then do;
		check_active_stus_2024 = -3; 
	end;
	else if year_2024 = 1 and missing(deact_date_2024) and missing(react_date_2024) then do;
		check_active_stus_2024 = 2;
	end;
	else if year_2024 = 1 and missing(deact_date_2024) and not missing(react_date_2024) then do;
		if react_date_2024 <= mdy(4, 30, 2024) then check_active_stus_2024 = 3; 
	end;
	active_npi_2024 = (check_active_stus_2024 in (1, 2, 3));
run;

/* Create flag for taxonomies to include in the sample frame using list of relevant taxonomies from 010 program, read in above */
proc sql;
	create table nppes_2021_2023_2024_w_taxincl as
	select a.*,
		   case
				when not missing(b.prvdr_txnmy_cd) then 1 
				else 0
		   end as tax_include_2021,
		   case
				when not missing(c.prvdr_txnmy_cd) then 1 
				else 0
		   end as tax_include_2023,
		   case
				when not missing(d.prvdr_txnmy_cd) then 1 
				else 0
		   end as tax_include_2024,
		   b.licensing_2021,
		   b.dm_specialty_2021,
		   c.licensing_2023,
		   c.dm_specialty_2023,
		   d.licensing_2024,
		   d.dm_specialty_2024
	from nppes_2021_2023_2024 as a
	left join taxonomy_include (rename = (licensing = licensing_2021 dm_specialty = dm_specialty_2021)) as b on a.taxonomy_2021 = b.prvdr_txnmy_cd
	left join taxonomy_include (rename = (licensing = licensing_2023 dm_specialty = dm_specialty_2023)) as c on a.taxonomy_2023 = c.prvdr_txnmy_cd
	left join taxonomy_include (rename = (licensing = licensing_2024 dm_specialty = dm_specialty_2024)) as d on a.taxonomy_2024 = d.prvdr_txnmy_cd;
quit;

/* Identify the most recent address for each NPI to geocode */
data nppes_2021_2023_2024_w_taxincl2;
	set nppes_2021_2023_2024_w_taxincl;
	priority = .;
	if not missing(prctc_zip_2024) then priority = 1;
	else if not missing(mlg_zip_2024) then priority = 2;
	else if not missing(prctc_zip_2023) then priority = 3;
	else if not missing(mlg_zip_2023) then priority = 4;
	else if not missing(prctc_zip_2021) then priority = 5;
	else if not missing(mlg_zip_2021) then priority = 6;
	else priority = 7;
run;

%let sources = prctc mlg prctc mlg prctc mlg;
%let yrs = 2024 2024 2023 2023 2021 2021;
%macro doit;
	data nppes_2021_2023_2024_w_taxincl3;
		set nppes_2021_2023_2024_w_taxincl2;
		%do i = 1 %to 6;
			%let source = %scan(&sources., &i.);
			%let yr = %scan(&yrs., &i.);
			if priority = &i. then do;
				priority_address_1 = &source._add_1_&yr.;
				priority_address_2 = &source._add_2_&yr.;
				priority_city = &source._city_&yr.;
				priority_state = &source._state_&yr.;
				priority_zip = &source._zip_&yr.;
				priority_cntry = &source._cntry_&yr.;
			end;
		%end;
	run;	 
%mend doit;
%doit;

proc sort data = nppes_2021_2023_2024_w_taxincl3 out = &outdata.;
	by npi;
run;

/* Export to a CSV file */
proc export data = &outdata. (where = (active_npi_2024 = 1 and tax_include_2024 = 1) 
							  keep = npi active_npi_2024 tax_include_2024 priority_address_1 priority_address_2 priority_city priority_state priority_zip priority_cntry) 
	outfile = "&csvoutdata."
	dbms = csv replace;
run;