/****************************************************************************************************************
   Program: 100_create_ffs_finders.sas

   Purpose: Flag FFS claims with dementia DX codes to create finder files of beneficiaries with dementia 
             for CY 2021 and FY 2023. Create files containing FQHC/RHC/CAHII claims in the two periods.

   Input files: _050_dementia_dx_codes.sas7bdat
                RIF(Q)yyyy.INPATIENT_CLAIMS_mm
                RIF(Q)yyyy.SNF_CLAIMS_mm
                RIF(Q)yyyy.HHA_CLAIMS_mm
                RIF(Q)yyyy.HOSPICE_CLAIMS_mm
                RIF(Q)yyyy.OUTPATIENT_CLAIMS_mm
                RIF(Q)yyyy.OUTPATIENT_REVENUE_mm
                RIF(Q)yyyy.BCARRIER_CLAIMS_mm
                RIF(Q)yyyy.BCARRIER_LINE_mm

   Output files: ndws_cc_bene_ff_ffs_by.sas7bdat
                 ndws_cc_bene_ff_ffs_ry.sas7bdat
                 ndws_op_spec3_ffs_clms_by.sas7bdat
                 ndws_op_spec3_ffs_clms_ry.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

libname in "<input directory>" access=readonly;
libname out "<output directory>";

%let tmplib = "<temporary output directory>";

%let base_yr = 2021;
%let rcnt_yr_ey = 2023;
%let rcnt_yr_em = 9;

%let rifq_sy = 2023;
%let rifq_sm = 1;

%let in_dx = in._050_dementia_dx_codes;
%let out_pfx = out.ndws_cc_bene_ff_ffs_;
%let out_op_pfx = out.ndws_op_spec3_ffs_clms_;

/*********************************************/
/* Put dementia diagnosis codes into a macro */
/*********************************************/
proc sql noprint;
   select quote(trim(code)) into :dmnt_dx separated by ',' from &in_dx.;
quit;

%put dmnt_dx = &dmnt_dx.;

/*********************************************/
/* Identify claims with dementia diagnoses   */
/*********************************************/

%macro process;

%do i = 1 %to 2;

%if &i. = 1 %then %do;
   %let start_yr = &base_yr.;
   %let start_mon = 1;
   %let yrtyp = by;
%end;
%else %do;
   %if &rcnt_yr_em. = 12 %then %do;
      %let start_yr = &rcnt_yr_ey.;
      %let start_mon = 1;
   %end;
   %else %do;
      %let start_yr = %sysevalf(&rcnt_yr_ey. - 1);
      %let start_mon = %sysevalf(&rcnt_yr_em. + 1);
   %end;
   %let yrtyp = ry;
%end;

%let totcnt = 12;

/* Set number of tasks (i.e., months) that will be compiled - here it will be equal to the totcnt (see above) */
%let task_compile=;
%do icnt = 1 %to &totcnt.;
   %let task_compile=&task_compile task&icnt.;
%end;

%put "***********Total number of gridding tasks - &totcnt.***********";
%put "***********List of gridding tasks - &task_compile.***********";

/* Grid Code: Grid Credentials */
OPTIONS METAUSER="&OUSER" METAPASS="&OPASSP";

/* Grid code: Enable Grid */
%LET CHG_QUEUE=QUEUE=normal; /*"normal" must be lower case*/
%LET RC=%SYSFUNC(GRDSVC_ENABLE(_ALL_, RESOURCE=sasCCW%str(;) JOBOPTS=CHG_QUEUE));

/* Loop through each task (i.e., month) */
%do cnt = 1 %to &totcnt.;

%let mon_init = %sysevalf(&start_mon. + &cnt. - 1);

%if %sysevalf(&mon_init. > 12,boolean) %then %do;
   %let mon = %sysevalf(&mon_init. - 12);
   %let yr = %sysevalf(&start_yr. + 1);
%end;
%else %do;
   %let mon = &mon_init.;
   %let yr = &start_yr;
%end;

%let mm = %sysfunc(putn(&mon., z2.));

%if &yr. > &rifq_sy. %then %do;
   %let riflib = RIFQ&yr.;
%end;
%else %if &yr. = &rifq_sy. %then %do;
   %if %sysevalf(&mon. >= &rifq_sm.,boolean) %then %do;
      %let riflib = RIFQ&yr.;
   %end;
   %else %do;
      %let riflib = RIF&yr.;
   %end;
%end;
%else %do;
   %let riflib = RIF&yr.;
%end;

/* Grid Code: Sign on to remote session for a task */
SIGNON TASK&cnt;

%syslput _all_;

%put ----> SUBMITTING: TASK&cnt. FOR YEAR &yr. MONTH &mm. using LIBRARY &riflib.;

/* Grid Code: Start remote submission for a task */
RSUBMIT TASK&cnt
CONNECTWAIT=NO /* Run tasks asynchronously */
CONNECTPERSIST=NO; /* Task end (i.e. sign off when complete) */

/* Flag inpatient, SNF, HHA, and hospice claims with dementia dx codes */

%macro inst(fname,typ);

   data &typ._dmnt_claims_m&cnt.(keep=bene_id dmnt);
      set &riflib..&fname._CLAIMS_&mm.(keep=bene_id icd_dgns_cd1-icd_dgns_cd25);

      dmnt = 0;

      array dx(25) icd_dgns_cd1-icd_dgns_cd25;

      do i = 1 to dim(dx);
         if missing(dx(i)) then leave;
         if dx(i) in (&dmnt_dx.) then do;
            dmnt = 1;
            leave;
         end;
      end;

      if dmnt then output;

   run;

   proc means data=&typ._dmnt_claims_m&cnt. noprint;
      class bene_id;
      types bene_id;
      var dmnt;
      output out=&tmplib..ndws_&typ._dmnt_benes_&yrtyp._m&cnt.(drop=_type_ _freq_) sum=&typ._n_dmnt_clms;
   run;

%mend inst;

%inst(INPATIENT,inp);
%inst(SNF,snf);
%inst(HHA,hha);
%inst(HOSPICE,hsp);

/*Identify outpatient claims from FQHCs, RHCs, type II CAHs, and flag outpatient claims with dementia dx codes */

data op_claims_m&cnt.(drop=clm_fac_type_cd clm_srvc_clsfctn_type_cd);
   set &riflib..OUTPATIENT_CLAIMS_&mm.(keep=bene_id clm_id clm_thru_dt icd_dgns_cd1-icd_dgns_cd25
                                            clm_fac_type_cd clm_srvc_clsfctn_type_cd
                                            at_physn_npi rndrng_physn_npi org_npi_num
                                            clm_srvc_fac_zip_cd);

   fqhc_rhc = (clm_fac_type_cd = "7" and clm_srvc_clsfctn_type_cd in ("1","3","7"));
   cah = (clm_fac_type_cd = "8" and clm_srvc_clsfctn_type_cd = "5");

   if fqhc_rhc or cah then output;

run;

data &tmplib..ndws_op_claims_&yrtyp._m&cnt.(keep=bene_id clm_thru_dt at_physn_npi rndrng_physn_npi
                                                 org_npi_num clm_srvc_fac_zip_cd
                                                 snf_visit homeoth_visit office_visit dmnt
                                                 fqhc_rhc cah_ii)
     op_dmnt_claims_m&cnt.(keep=bene_id clm_thru_dt dmnt);
   merge op_claims_m&cnt.(in=a)
         &riflib..OUTPATIENT_REVENUE_&mm.(keep=bene_id clm_id rev_cntr);
   by bene_id clm_id;

   retain prf_srvc snf_visit homeoth_visit;

   array dx(25) icd_dgns_cd1-icd_dgns_cd25;

   if a then do;

      if first.clm_id then do;
         prf_srvc = 0;
         snf_visit = 0;
         homeoth_visit = 0;
      end;

      if substrn(rev_cntr,1,3) in ("096","097","098") then prf_srvc = 1;
      else if rev_cntr in ("0524","0525") then snf_visit = 1;
      else if rev_cntr in ("0522","0527","0528") then homeoth_visit = 1;

      if last.clm_id then do;

         cah_ii = (cah and prf_srvc);
         if snf_visit then homeoth_visit = 0;
         office_visit = not (snf_visit or homeoth_visit);

         dmnt = 0;

         do i = 1 to dim(dx);
            if missing(dx(i)) then leave;
            if dx(i) in (&dmnt_dx.) then do;
               dmnt = 1;
               leave;
            end;
         end;

         if fqhc_rhc or cah_ii then do;
            output &tmplib..ndws_op_claims_&yrtyp._m&cnt.;
            if dmnt then output op_dmnt_claims_m&cnt.;
         end;

      end;

   end;

run;

/* Read in Carrier claims excluding ambulance BETOS codes in order to limit Carrier claims 
   in the next step to those with at least one non-ambulance BETOS code */ 
proc sql noprint;
   create table crr_nonamb_clms_m&cnt. as
   select bene_id, clm_id
   from &riflib..BCARRIER_LINE_&mm.
   where betos_cd ne "O1A"
   group by bene_id, clm_id
   order by bene_id, clm_id;
quit;

/* Pull non-DME Carrier claims (claim type code = 71) and limit to those with at least
   one non-ambulance BETOS code, and flag claims with dementia dx codes */
data crr_dmnt_claims_m&cnt.(keep=bene_id clm_thru_dt dmnt);
   merge &riflib..BCARRIER_CLAIMS_&mm.(in=a keep=bene_id clm_id nch_clm_type_cd clm_thru_dt
                                                 icd_dgns_cd1-icd_dgns_cd12
                                            where=(nch_clm_type_cd = "71"))
         crr_nonamb_clms_m&cnt.(in=b);
   by bene_id clm_id;
   if a and b;

   dmnt = 0;

   array dx(12) icd_dgns_cd1-icd_dgns_cd12;

   do i = 1 to dim(dx);
      if missing(dx(i)) then leave;
      if dx(i) in (&dmnt_dx.) then do;
         dmnt = 1;
         leave;
      end;
   end;

   if dmnt then output;

run;

/* Stack carrier and outpatient claims with dementia dx codes and find earliest and latest date for dementia diagnoses */
data crrop_dmnt_claims_m&cnt.;
   set crr_dmnt_claims_m&cnt.(in=a) op_dmnt_claims_m&cnt.(in=b);
run;

proc means data=crrop_dmnt_claims_m&cnt. noprint;
   class bene_id;
   types bene_id;
   var dmnt clm_thru_dt;
   output out=&tmplib..ndws_crrop_dmnt_benes_&yrtyp._m&cnt.(drop=_type_ _freq_)
          sum(dmnt)=crrop_n_dmnt_clms
          min(clm_thru_dt)=first_date
          max(clm_thru_dt)=last_date;
run;

/* Grid code: End remote submission for a task */
ENDRSUBMIT;

%put TASK&cnt CODE RAN ON MACHINE %sysfunc(GRDSVC_GETNAME(TASK&cnt));

%end; /* End task loop */

/* Grid code: Wait for all grid tasks to complete and sign off */
WAITFOR _ALL_ &task_compile;
SIGNOFF _ALL_;

/*****************************************************************************************************/
/* Combine months of inpatient, SNF, HHA, and hospice claims into annual claim files and sum the number 
of dementia claims for each beneficiary */
/*****************************************************************************************************/
%macro inst_stack(typ);

   data &typ._dmnt_benes_&yrtyp._allmths;
      set &tmplib..ndws_&typ._dmnt_benes_&yrtyp._m1-&tmplib..ndws_&typ._dmnt_benes_&yrtyp._m12;
      by bene_id;
   run;

   proc datasets lib=&tmplib. nolist;
      delete ndws_&typ._dmnt_benes_&yrtyp._m:;
   run;

   proc means data=&typ._dmnt_benes_&yrtyp._allmths noprint;
      by bene_id;
      var &typ._n_dmnt_clms;
      output out=&typ._dmnt_benes_&yrtyp.(drop=_type_ _freq_) sum=;
   run;

%mend inst_stack;

%inst_stack(inp);
%inst_stack(snf);
%inst_stack(hha);
%inst_stack(hsp);

/********************************************************************************/
/* Combine months of FQHC/RHC/CAHII claims and output for downstream processing */
/********************************************************************************/

data &out_op_pfx.&yrtyp.;
   set &tmplib..ndws_op_claims_&yrtyp._m1-&tmplib..ndws_op_claims_&yrtyp._m12;
   by bene_id;
run;

proc datasets lib=&tmplib. nolist;
   delete ndws_op_claims_&yrtyp._m:;
run;

/********************************************************************************/
/* Combine months of Carrier FQHC/RHC/CAHII claims with dementia and limit to   */
/* beneficiaries with 2 or more dementia claims at least 7 days apart.          */
/********************************************************************************/

data crrop_dmnt_benes_&yrtyp._allmths;
   set &tmplib..ndws_crrop_dmnt_benes_&yrtyp._m1-&tmplib..ndws_crrop_dmnt_benes_&yrtyp._m12;
   by bene_id;
run;

proc datasets lib=&tmplib. nolist;
   delete ndws_crrop_dmnt_benes_&yrtyp._m:;
run;

proc means data=crrop_dmnt_benes_&yrtyp._allmths noprint;
   by bene_id;
   var crrop_n_dmnt_clms first_date last_date;
   output out=crrop_dmnt_benes_&yrtyp.(drop=_type_ _freq_)
          sum(crrop_n_dmnt_clms)=crrop_n_dmnt_clms
          min(first_date)=first_date
          max(last_date)=last_date;
run;

data crrop_dmnt_benes_&yrtyp.(drop=first_date last_date);
   set crrop_dmnt_benes_&yrtyp.;
   if last_date - first_date >= 7 then output;
run;

/********************************************************************************/
/* Merge dementia beneficiary-level files and output finder file                */
/********************************************************************************/

data &out_pfx.&yrtyp.;
   merge inp_dmnt_benes_&yrtyp.(in=inp)
         snf_dmnt_benes_&yrtyp.(in=snf)
         hha_dmnt_benes_&yrtyp.(in=hha)
         hsp_dmnt_benes_&yrtyp.(in=hsp)
         crrop_dmnt_benes_&yrtyp.(in=crrop);
   by bene_id;

   if not inp then inp_n_dmnt_clms = 0;
   if not snf then snf_n_dmnt_clms = 0;
   if not hha then hha_n_dmnt_clms = 0;
   if hsp then hspc = 1;
   else do;
      hspc = 0;
      hsp_n_dmnt_clms = 0;
   end;
   if not crrop then crrop_n_dmnt_clms = 0;

   ffs_clm = 1;

run;

proc sort data=&out_pfx.&yrtyp. out=dupchk_&yrtyp. nodupkey;
   by bene_id;
run;

%end;

%mend process;

%process;