/****************************************************************************************************************
   Program: 101_flag_dementia.sas

   Purpose: For inpatient, SNF, HHA, outpatient, and carrier encounter files, flag
            dementia diagnoses. For the outpatient encounter file, create a subset 
            file with all FQHC, RHC, and CAH II encounters.

   Input files: ENRFPL21.IP_BASE_ENC
                ENRFPL21.SNF_BASE_ENC
                ENRFPL21.HHA_BASE_ENC
                ENRFPL21.OP_BASE_ENC
                ENRFPL21.OP_REVENUE_ENC
                ENRFPL21.CARRIER_BASE_ENC
				_050_dementia_dx_codes.sas7bdat

   Output files: _101_ip_dmnt_flgd_2021.sas7bdat
                 _101_snf_dmnt_flgd_2021.sas7bdat
                 _101_hha_dmnt_flgd_2021.sas7bdat
                 _101_op_dmnt_flgd_2021.sas7bdat
                 _101_op_dmnt_flgd_frc_2021.sas7bdat
                 _101_car_dmnt_flgd_2021.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

libname lookups "<input directory>" access = readonly;
libname out "<output directory>";

%let dmnt_cd_lookup = lookups._050_dementia_dx_codes;
%let yr = 2021;
%let yrshrt = 21;

/*********************************************/
/* Put dementia diagnosis codes into a macro */
/*********************************************/
title "Dementia Diagnosis Codes for Flagging";
proc sql;
	select distinct quote(trim(left(code))) into :dmnt_cds separated by ", "
	from &dmnt_cd_lookup.;
quit;

/*************************************************************************************************************************************/
/* Macro to read-in different MA encounter files, add diagnosis / dementia flags and vars, and other OP/carrier processing           */
/*************************************************************************************************************************************/
%macro preprocess_ma_encounters(file, num_dgnscds, rerun_outdata1=1);

	%if &file. = car %then %do;
		%let file2 = carrier;
	%end;
	%else %do;
		%let file2 = &file.;
	%end;

	/* Setup output dataset name macros */
	%let outdata1 = out._101_&file._dmnt_flgd_&yr.;
	%if &file. = op %then %let outdata2 = out._101_&file._dmnt_flgd_frc_&yr.;

	/* Only rerun to create outdata1 if macro flag indicates to do so */
	%if &rerun_outdata1. = 1 %then %do;

		/* Read in MA encounters for each file type and flag dementia diagnoses */
		data &outdata1.;
			set enrfpl&yrshrt..&file2._base_enc;
			dement = 0;
			array dgns_cds {&num_dgnscds.} icd_dgns_cd1 - icd_dgns_cd&num_dgnscds.;
			do i = 1 to &num_dgnscds.;
				if dgns_cds{i} in (&dmnt_cds.) then dement = 1;
			end;
			drop i;
		run;

	%end;
		
	/* For OP encounters, create a dataset with just FQHC, RHC, and CAH II encounters */
	/* Pull all revenue center information, flag revenue center lines, and merge back to header data */
	/* Add some additional flags */
	%if &file. = op %then %do;

		/* Limit to FQHC, RHC, and CAH encounters */
		data &file._dmnt_flgd_frc_&yr.;
			set &outdata1. (where = (clm_type_cd in ("4071", "4077", "4085")));
		run;

		/* Pull revenue center lines for FQHC, RHC, and CAH encounters */
		proc sql;
			create table &file._dmnt_flgd_frc_lns_&yr. as
			select a.bene_id,
				   a.clm_type_cd,
				   a.enc_join_key,
				   case
				   		when substr(a.rev_cntr, 1, 3) in ("096", "097", "098") then 1
						else 0
				   end as prf_srvc,
				   case
				   		when a.rev_cntr in ("0524", "0525") then 1
						else 0
				   end as snf_visit,
				   case
				   		when a.rev_cntr in ("0522", "0527", "0528") then 1
						else 0
				   end as homeoth_visit
			from enrfpl&yrshrt..&file._revenue_enc as a
			inner join
			&file._dmnt_flgd_frc_&yr. as b
			on a.bene_id = b.bene_id and a.clm_type_cd = b.clm_type_cd and a.enc_join_key = b.enc_join_key;
		quit;

		/* Aggregate the revenue center lines for FQHC, RHC, and CAH encounters and merge back on to header encounters */
		proc sql;
			create table &file._dmnt_flgd_frc_agg_lns_&yr. as
			select a.*,
				   b.prf_srvc,
				   b.snf_visit,
				   b.homeoth_visit,
				   case
				   		when missing(b.enc_join_key) then 1
						when missing(a.enc_join_key) then 2
						when not missing(a.enc_join_key) and not missing(b.enc_join_key) then 3
						else .
					end as _merge
			from &file._dmnt_flgd_frc_&yr. as a
			full join
			(select bene_id,
				    clm_type_cd,
				    enc_join_key,
				    max(prf_srvc) as prf_srvc,
				    max(snf_visit) as snf_visit,
				    max(homeoth_visit) as homeoth_visit
			 from &file._dmnt_flgd_frc_lns_&yr.
			 group by bene_id, clm_type_cd, enc_join_key) as b
			on a.bene_id = b.bene_id and a.clm_type_cd = b.clm_type_cd and a.enc_join_key = b.enc_join_key;
		quit;

		/* Additional cleaning and creation of flags */
		data &outdata2.;
			set &file._dmnt_flgd_frc_agg_lns_&yr.;

			* if no lines associated with header, set all aggregated line variables to 0;
			if _merge = 1 then do; 
				prf_srvc = 0;
				snf_visit = 0;
				homeoth_visit = 0;
			end;

 			*drop claims from CAHs where professionals bill separately;
			cah_II = (prf_srvc = 1 and clm_type_cd = "4085");
			if cah_II = 0 and clm_type_cd = "4085" then delete;

			* office visits flag;
			if snf_visit = 1 and homeoth_visit = 1 then homeoth_visit = 0;
			office_visit = (snf_visit = 0 and homeoth_visit = 0); 

			rename at_physn_npi = npi;
			drop _merge;
		run;

	%end;

%mend preprocess_ma_encounters;

/***************/
/* Call macros */
/***************/
%preprocess_ma_encounters(file=ip, num_dgnscds=25);
%preprocess_ma_encounters(file=snf, num_dgnscds=25);
%preprocess_ma_encounters(file=hha, num_dgnscds=25);
%preprocess_ma_encounters(file=op, num_dgnscds=25);
%preprocess_ma_encounters(file=car, num_dgnscds=13);