/****************************************************************************************************************
   Program: 102_bene_lvl_dementia.sas

   Purpose: Combine the inpatient, SNF, HHA, FQHC/RHC/CAHII and Carrier encounter files with dementia diagnoses
              and apply the Bynum-Standard algorithm (at least one inpatient, SNF, HHA claims or 2 or more 
			  carrier/FQHC/RHC/CAHII claims at least 7 days apart) to identify beneficiaries with dementia
              in encounter data.

   Input files: _101_ip_dmnt_flgd_2021.sas7bdat
                _101_snf_dmnt_flgd_2021.sas7bdat
                _101_hha_dmnt_flgd_2021.sas7bdat
                _101_op_dmnt_flgd_frc_2021.sas7bdat
                _101_car_dmnt_flgd_2021.sas7bdat

   Output files: _102_ipsnfhha_dmnt_bene_lvl_2021.sas7bdat
                 _102_opcar_dmnt_bene_lvl_2021.sas7bdat
                 _102_final_dmnt_bene_lvl_2021.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

libname in "<input director>" access = readonly;
libname out "<output director>";

%let yr = 2021;
%let indata_ip = in._101_ip_dmnt_flgd_&yr.;
%let indata_snf = in._101_snf_dmnt_flgd_&yr.;
%let indata_hha = in._101_hha_dmnt_flgd_&yr.;
%let indata_op_frc = in._101_op_dmnt_flgd_frc_&yr.;
%let indata_car = in._101_car_dmnt_flgd_&yr.;
%let outdata_ipsnfhha = out._102_ipsnfhha_dmnt_bene_lvl_&yr.;
%let outdata_opcar = out._102_opcar_dmnt_bene_lvl_&yr.;
%let outdata_ipsnfhhaopcar = out._102_final_dmnt_bene_lvl_&yr.;

/***********************************************************************/
/* Macro for limiting each claim type to final action dementia records */
/***********************************************************************/
%macro prep_ma_data(file, filevar);

	/* Limit MA encounters to final action and dementia */
	data &file.;
		set &&indata_&file.. (where = (clm_finl_actn_ind = "Y" and dement = 1));
		ma_&filevar._n_dmnt_clms = 1;
	run;

%mend prep_ma_data;

/******************************************************************************************************/
/* Macro for finding dementia in inpatient, SNF, and HHA encounters and aggregating to the bene-level */
/******************************************************************************************************/
%macro ipsnfhha_benelvl;

	/* Append inpatient, SNF, and HHA MA datasets already limited to dementia final action ecounters */
	data ipsnfhha_&yr.;
		set ip
			snf
			hha
		;
	run;

	/* Create bene-level dementia flags */
	proc sql;
		create table &outdata_ipsnfhha. as 
		select bene_id,
			   sum(ma_inp_n_dmnt_clms) as ma_inp_n_dmnt_clms,
			   sum(ma_snf_n_dmnt_clms) as ma_snf_n_dmnt_clms,
			   sum(ma_hha_n_dmnt_clms) as ma_hha_n_dmnt_clms,
			   max(dement) as dement /* note this will always be 1 since we have already limited to dementia encounters */
		from ipsnfhha_&yr.
		group by bene_id;
	quit;

%mend ipsnfhha_benelvl;

/****************************************************************************************************/
/* Macro for finding dementia in outpatient and carrier encounters and aggregating to the bene-level */
/****************************************************************************************************/
%macro opcar_benelvl;

	/* Append outpatient FQHC/RHC/CAH II and carrier MA datasets already limited to dementia final action ecounters */
	data opcar_&yr.;
		set op_frc
			car
		;
	run;

	/* Limit to benes with 2 or more dementia encounters at least 7 days apart (bene-level dataset) */
	proc sql;
		create table &outdata_opcar. as
		select bene_id,
			   count(*) as ma_crrop_n_dmnt_clms,
			   min(clm_thru_dt) as first_date format = date9.,
			   max(clm_thru_dt) as last_date format = date9.,
			   1 as dement /* can do this here since we already limited to dementia encounters */
		from opcar_&yr.
		group by bene_id
		having ma_crrop_n_dmnt_clms >= 2 and intck("day", first_date, last_date) >= 7;
	quit;
	
%mend opcar_benelvl;

/*****************************************************************************************************/
/* Macro for combining and deduping the inpatient/SNF/HHA and outpatient/carrier bene-level datasets */
/*****************************************************************************************************/
%macro ipsnfhhaopcar_benelvl;

	/* Combine the inpatient/SNF/HHA and outpatient/carrier bene-level datasets */
	data ipsnfhhaopcar_benelvl_&yr.;
		set &outdata_ipsnfhha.
			&outdata_opcar.
		;
	run;

	proc sql;
		create table ipsnfhhaopcar_benelvl2_&yr. as
		select bene_id,
			   sum(ma_inp_n_dmnt_clms) as ma_inp_n_dmnt_clms,
			   sum(ma_snf_n_dmnt_clms) as ma_snf_n_dmnt_clms,
			   sum(ma_hha_n_dmnt_clms) as ma_hha_n_dmnt_clms,
			   sum(ma_crrop_n_dmnt_clms) as ma_crrop_n_dmnt_clms,
			   1 as ma_enctr
		from ipsnfhhaopcar_benelvl_&yr.
		group by bene_id;
	quit;

	data &outdata_ipsnfhhaopcar.;
		set ipsnfhhaopcar_benelvl2_&yr.;
		* replace missing claim counts with 0s (indicates a bene does not have any claims of the given type);
		if missing(ma_inp_n_dmnt_clms) then ma_inp_n_dmnt_clms = 0;
		if missing(ma_snf_n_dmnt_clms) then ma_snf_n_dmnt_clms = 0;
		if missing(ma_hha_n_dmnt_clms) then ma_hha_n_dmnt_clms = 0;
		if missing(ma_crrop_n_dmnt_clms) then ma_crrop_n_dmnt_clms = 0;
	run;

%mend ipsnfhhaopcar_benelvl;

/***************/
/* Call macros */
/***************/
%prep_ma_data(file=ip, filevar=inp);
%prep_ma_data(file=snf, filevar=snf);
%prep_ma_data(file=hha, filevar=hha);
%prep_ma_data(file=car, filevar=car);
%prep_ma_data(file=op_frc, filevar=op);
%ipsnfhha_benelvl;
%opcar_benelvl;
%ipsnfhhaopcar_benelvl;