/****************************************************************************************************************
   Program: 210_find_npis.sas

   Purpose: Find all the clinicians who treated FFS beneficiaries with dementia in CY 2021 and FY 2023. This 
            program pulls FFS claims for beneficiaries identified as having dementia in FFS claims.

   Input files: _050_dementia_dx_codes.sas7bdat
                ndws_cc_bene_ff_mbsf_by.sas7bdat
                ndws_cc_bene_ff_mbsf_ffs_ry.sas7bdat
                ndws_op_spec3_ffs_clms_by.sas7bdat
                ndws_op_spec3_ffs_clms_ry.sas7bdat
                RIF(Q)yyyy.BCARRIER_CLAIMS_mm
                RIF(Q)yyyy.BCARRIER_LINE_mm
                PDEyyyy.PDE_ENCRYPT_LINK_yyyy_mm

   Output files: ndws_cc_ffs_npis_by.sas7bdat
                 ndws_cc_ffs_npis_ry.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

%let prognum = 210;

libname in_dx "<dementia dx directory>" access=readonly;
libname in_ff "<finder file directory>" access=readonly;
libname in_s3 "<ffs directory>" access=readonly;
libname out "<output directory>";

%let tmplib = "<temporary output directory>";

%let base_yr = 2021;
%let rcnt_yr_ey = 2023;
%let rcnt_yr_em = 9;
%let ltst_pde = 2022;

%let rifq_sy = 2023;
%let rifq_sm = 1;

%let in_dx = in_dx._050_dementia_dx_codes;
%let in_ff_pfx = in_ff.ndws_cc_bene_ff_mbsf_;
%let in_s3_pfx = in_s3.ndws_op_spec3_ffs_clms_;
%let out_pfx = out.ndws_cc_ffs_npis_;

/*********************************************/
/* Put dementia diagnosis codes into a macro */
/*********************************************/

proc sql noprint;
   select quote(trim(code)) into :dmnt_dx separated by ',' from &in_dx.;
quit;

%put dmnt_dx = &dmnt_dx.;

/********************************************************/
/* Identify NPIs who treated beneficiaries with dementia */
/********************************************************/

%macro process;

%do i = 1 %to 2;

%if &i. = 1 %then %do;
   %let start_yr = &base_yr.;
   %let start_mon = 1;
   %let yrtyp = by;
   %let ff_sfx = &yrtyp.;
   %let s3_sfx = &yrtyp.;
   %let out_sfx = &yrtyp.;
%end;
%else %do;
   %if &rcnt_yr_em. = 12 %then %do;
      %let start_yr = &rcnt_yr_ey.;
      %let start_mon = 1;
   %end;
   %else %do;
      %let start_yr = %sysevalf(&rcnt_yr_ey. - 1);
      %let start_mon = %sysevalf(&rcnt_yr_em. + 1);
   %end;
   %let yrtyp = ry;
   %let ff_sfx = ffs_&yrtyp.;
   %let s3_sfx = &yrtyp.;
   %let out_sfx = &yrtyp.;
%end;

%let totcnt = 12;

/* Set number of tasks (i.e., months) that will be compiled - here it will be equal to the totcnt (see above) */
%let task_compile=;
%do icnt = 1 %to &totcnt.;
   %let task_compile=&task_compile task&icnt.;
%end;

%put "***********Total number of gridding tasks - &totcnt.***********";
%put "***********List of gridding tasks - &task_compile.***********";

/* Grid Code: Grid Credentials */
OPTIONS METAUSER="&OUSER" METAPASS="&OPASSP";

/* Grid code: Enable Grid */
%LET CHG_QUEUE=QUEUE=normal; /*"normal" must be lower case*/
%LET RC=%SYSFUNC(GRDSVC_ENABLE(_ALL_, RESOURCE=sasCCW%str(;) JOBOPTS=CHG_QUEUE));

/* Loop through each task (i.e., month)*/
%do cnt = 1 %to &totcnt.;

%let mon_init = %sysevalf(&start_mon. + &cnt. - 1);

%if %sysevalf(&mon_init. > 12,boolean) %then %do;
   %let mon = %sysevalf(&mon_init. - 12);
   %let yr = %sysevalf(&start_yr. + 1);
%end;
%else %do;
   %let mon = &mon_init.;
   %let yr = &start_yr;
%end;

%let mm = %sysfunc(putn(&mon., z2.));

%if &yr. > &rifq_sy. %then %do;
   %let riflib = RIFQ&yr.;
%end;
%else %if &yr. = &rifq_sy. %then %do;
   %if %sysevalf(&mon. >= &rifq_sm.,boolean) %then %do;
      %let riflib = RIFQ&yr.;
   %end;
   %else %do;
      %let riflib = RIF&yr.;
   %end;
%end;
%else %do;
   %let riflib = RIF&yr.;
%end;

/* Grid Code: Sign on to remote session for a task */
SIGNON TASK&cnt;

%syslput _all_;

%put ----> SUBMITTING: TASK&cnt. FOR YEAR &yr. MONTH &mm.;

/* Grid Code: Start remote submission for a task */
RSUBMIT TASK&cnt
CONNECTWAIT=NO /* Run tasks asynchronously */
CONNECTPERSIST=NO; /* Task end (i.e. sign off when complete) */

libname in_ff "<finder file directory>" access=readonly;

/**********************************************************************************/
/* Step 1: Process Carrier Data                                                   */
/*                                                                                */
/* Pull carrier header and lines for the FFS beneficiaries with dementia to       */
/* identify the NPIs who treated these beneficiaries (based on (performing        */
/* physician NPI field).                                                          */
/*																				  */
/* Create a flag indicating whether the NPI diagnosed the beneficiary with        */
/* dementia and flags for settings (outpatient, residential, inpatient, other,    */
/* or Part D prescriber (set to zero at this stage)) where the NPIs saw the 	  */
/* beneficiaries based on the place of service variable.					      */
/*																				  */
/* All line level indicators (performing NPI, place of service, and physician     */
/* practice zip code are taken from the first line on the claim.                  */
/*                                                                                */
/* Beneficiary eligibility and demographic information from the finder file is    */
/* merged onto each claim.                                                        */
/**********************************************************************************/

data carrier_claims_m&cnt.(keep=bene_id clm_id dmnt black hispanic white alloths any_lis dual dual_or_lis);
   merge &in_ff_pfx.&ff_sfx.(in=a keep=bene_id ffs_clm black hispanic white alloths any_lis dual dual_or_lis
                                  where=(ffs_clm))
         &riflib..BCARRIER_CLAIMS_&mm.(in=b keep=bene_id clm_id icd_dgns_cd1-icd_dgns_cd12);
   by bene_id;
   if a and b;

   dmnt = 0;

   array dx(12) icd_dgns_cd1-icd_dgns_cd12;

   do i = 1 to dim(dx);
      if missing(dx(i)) then leave;
      if dx(i) in (&dmnt_dx.) then do;
         dmnt = 1;
         leave;
      end;
   end;

run;

data carrier_lines_m&cnt.(keep=bene_id clm_id line_pos npi physn_zip_cd);
   merge &in_ff_pfx.&ff_sfx.(in=a keep=bene_id ffs_clm where=(ffs_clm))
         &riflib..BCARRIER_LINE_&mm.(in=b keep=bene_id clm_id
                                               prf_physn_npi line_place_of_srvc_cd physn_zip_cd
                                          rename=(prf_physn_npi = npi
                                                  line_place_of_srvc_cd = line_pos
                                                  physn_zip_cd = zip_orig));
   by bene_id;
   if a and b;

   length physn_zip_cd $ 9;

   physn_zip_cd = strip(zip_orig);

run;

data carrier_line_info_m&cnt.;
   set carrier_lines_m&cnt.;
   by bene_id clm_id;
   if first.clm_id then output;
run;

data &tmplib..ndws_&prognum._carrier_claims_&yrtyp._m&cnt.(keep=has_line_info pos_: dmnt npi physn_zip_cd
                                                                bene_id black hispanic white alloths any_lis
                                                                dual dual_or_lis);
   merge carrier_claims_m&cnt.(in=a) carrier_line_info_m&cnt.(in=b);
   by bene_id clm_id;
   if a;

   has_line_info = b;

   pos_outpt = 0;
   pos_res = 0;
   pos_inpt = 0;
   pos_elswhr = 0;
   pos_partd = 0;

   if line_pos in ("05","07","10","11","17","19","20","22","49","50","62","71","72") then pos_outpt = 1;
   else if line_pos in ("13","14","31","32","33","34","54") then pos_res = 1;
   else if line_pos in ("06","08","21","61") then pos_inpt = 1;
   else pos_elswhr = 1;

run;

/**********************************************************************************/
/* Step 2: Process Part D Drug Event Data                                         */
/*                                                                                */
/* Pull Part D claims with for beneficiaries with dementia to identify and create */
/* the same setting flags as carrier (outpatient, residential, inpatient, other,  */
/* and Part D prescriber). All settings except Part D prescriber are set to zero. */
/*                                                                                */
/* Beneficiary eligibility and demographic information from the finder file is    */
/* merged onto each claim.                                                        */
/**********************************************************************************/

%macro ptd;

%if &yr. <= &ltst_pde. %then %do;

data &tmplib..ndws_&prognum._ptd_claims_&yrtyp._m&cnt.(keep=pos_: npi
                                                            bene_id black hispanic white alloths any_lis
                                                            dual dual_or_lis);
   merge &in_ff_pfx.&ff_sfx.(in=a keep=bene_id ffs_clm black hispanic white alloths any_lis dual dual_or_lis
                                  where=(ffs_clm))
         PDE&yr..PDE_ENCRYPT_LINK_&yr._&mm.(in=b keep=bene_id prscrbr_id prscrbr_id_qlfyr_cd
                                                 where=(not (missing(prscrbr_id) or
                                                             missing(prscrbr_id_qlfyr_cd) or
                                                             prscrbr_id_qlfyr_cd in ("06","08","12"))));
   by bene_id;
   if a and b;

   length npi $ 12;

   npi = strip(prscrbr_id);

   pos_outpt = 0;
   pos_res = 0;
   pos_inpt = 0;
   pos_elswhr = 0;
   pos_partd = 1;

run;

%end;

%mend ptd;

%ptd;

/* Grid code: End remote submission for a task */
ENDRSUBMIT;

%put TASK&cnt CODE RAN ON MACHINE %sysfunc(GRDSVC_GETNAME(TASK&cnt));

%end; /* End task loop */

/* Grid code: Wait for all grid tasks to complete and sign off */
WAITFOR _ALL_ &task_compile;
SIGNOFF _ALL_;

/**********************************************************************************/
/* Combine all months of Carrier claims pulled for beneficiaries with dementia    */
/**********************************************************************************/

data carrier_claims_&yrtyp.;
   set &tmplib..ndws_&prognum._carrier_claims_&yrtyp._m:;
   by bene_id;
run;

/**********************************************************************************/
/* Combine all months of Part D event data pulled for beneficiaries with dementia */
/**********************************************************************************/

data ptd_claims_&yrtyp.;
   set &tmplib..ndws_&prognum._ptd_claims_&yrtyp._m:;
   by bene_id;
run;

proc datasets lib=&tmplib. nolist;
   delete ndws_&prognum._carrier_claims_&yrtyp._m:
          ndws_&prognum._ptd_claims_&yrtyp._m:;
run;

/**********************************************************************************/
/* Step 3: Process FQHC, RHC, Type II CAH claims                                  */
/*                                                                                */
/* Limit the FQHC, RHC, Type II CAH claims from the 100 program to those for      */
/* beneficiaries with dementia by merging with the finder file. Rename the        */
/* setting variables so we can stack with Carrier and Part D claims pulled above  */
/* and set the inpatient and Part D prescribing settings to zero. Assign an NPI to*/
/* the claim as the attending NPI if not missing, otherwise use the rendering or  */
/* organizational NPI. Note, the attending NPI was almost always non-missing.     */
/*                                                                                */
/* Keep the claim dementia flag created in the 100 program, the facility zip      */
/* code, NPI, place of service indicators, and the beneficiary eligibility and    */
/* demographic information from the finder file.                                  */
/**********************************************************************************/

data fqhcrhccah_claims_&yrtyp.(keep=pos_: dmnt npi physn_zip_cd
                                    bene_id black hispanic white alloths any_lis dual dual_or_lis);
   merge &in_ff_pfx.&ff_sfx.(in=a keep=bene_id ffs_clm black hispanic white alloths any_lis dual dual_or_lis
                                  where=(ffs_clm))
         &in_s3_pfx.&s3_sfx.(in=b keep=bene_id dmnt snf_visit homeoth_visit office_visit
                                       at_physn_npi rndrng_physn_npi org_npi_num clm_srvc_fac_zip_cd
                                  rename=(snf_visit = pos_res
                                          homeoth_visit = pos_elswhr
                                          office_visit = pos_outpt
                                          clm_srvc_fac_zip_cd = physn_zip_cd));
   by bene_id;
   if a and b;

   length npi $ 12;

   if missing(at_physn_npi) then do;
      if missing(rndrng_physn_npi) then npi = strip(org_npi_num);
      else npi = strip(rndrng_physn_npi);
   end;
   else npi = strip(at_physn_npi);

   pos_inpt = 0;
   pos_partd = 0;

run;

/**********************************************************************************/
/* Step 4: Stack Carrier, Part D, FQHC/RHC/Type II CAH claims and roll up to the  */
/* NPI level.                                                                     */
/*                                                                                */
/* The steps below will roll-up the Carrier, Part D, FQHC/RHC/Type II CAH claims  */
/* for beneficiaries with dementia to the NPI level, creating variables for the   */
/* settings where each NPI provided care to their dementia patients, practice zip */
/* code(s) (up to 10), counts of the number of beneficiaries with dementia        */
/* served (total and number dually eligible or eligible for a low-income drug 	  */
/* subsidy), and the number of beneficiaries who each NPI diagnosed with dementia.*/                            
/**********************************************************************************/

data all_ffs_claims_&yrtyp.(drop=has_line_info physn_zip_cd num_zip_claims pos_:)
     ptb_claim_zips_&yrtyp.(keep=npi physn_zip_cd num_zip_claims);
   set carrier_claims_&yrtyp.(in=a where=(has_line_info and not missing(npi)))
       fqhcrhccah_claims_&yrtyp.(in=b where=(not missing(npi)))
       ptd_claims_&yrtyp.(in=c);

   carrier = a;
   fqhcrhccah = b;
   partd = c;

   num_zip_claims = 1;

   if c then dmnt = 0;

   /* Set claim level flags for counting claims by place of service in the step below */
   ffs_clmcnt_res = pos_res;
   ffs_clmcnt_inpt = pos_inpt;
   ffs_clmcnt_outpt = pos_outpt;
   ffs_clmcnt_elswhr = pos_elswhr;
   ffs_rxcnt_partd = pos_partd;

   /* These flags will be used to count claims from the Carrier file by place of 
      service in the step below */
   ffs_ptb_pos_res = carrier and pos_res;
   ffs_ptb_pos_inpt = carrier and pos_inpt;
   ffs_ptb_pos_outpt = carrier and pos_outpt;
   ffs_ptb_pos_elswhr = carrier and pos_elswhr;

   /* Output a file containing practice zip codes for identifying the top 10 billed by the NPI */
   if (a or b) and not missing(physn_zip_cd) then output ptb_claim_zips_&yrtyp.;

   /* Output all claims from Carrier, FQHC/RHC/Type II CAH, and Part D for rolling up to the NPI level */
   output all_ffs_claims_&yrtyp.;

run;

/* Count the number of claims by setting for each NPI */
proc means data=all_ffs_claims_&yrtyp. noprint;
   class npi;
   types npi;
   var ffs_:;
   output out=npi_ffs_clm_cnts_&yrtyp.(drop=_type_ _freq_) sum=;
run;

/* Count the number of claims billed to each practice zip code by NPI */
proc means data=ptb_claim_zips_&yrtyp. noprint;
   class npi physn_zip_cd;
   types npi*physn_zip_cd;
   var num_zip_claims;
   output out=ptb_zip_claim_cnts_&yrtyp.(drop=_type_ _freq_) sum=;
run;

proc sort data=ptb_zip_claim_cnts_&yrtyp.;
   by npi descending num_zip_claims;
run;

/* Identify up to 10 most common practice zip codes the NPI billed to */
data npi_ptb_zips_long_&yrtyp.;
   set ptb_zip_claim_cnts_&yrtyp.(drop=num_zip_claims);
   by npi;
   retain num_zips;

   if first.npi then num_zips = 1;
   else num_zips = num_zips + 1;

   if num_zips <= 10 then output;

run;

/* Array the top 10 zip codes in order of most billed to to least for each NPI */
proc transpose data=npi_ptb_zips_long_&yrtyp.
               out=npi_ptb_zips_&yrtyp.(drop=_label_ _name_) prefix=ptb_zip_;
    by npi;
    id num_zips;
    var physn_zip_cd;
run;

/* Merge the top 10 zip codes to the NPI level file with claim counts by setting 
   and assign the setting variables */
data npi_ffs_clm_flgs_&yrtyp.;
   merge npi_ffs_clm_cnts_&yrtyp. npi_ptb_zips_&yrtyp.;
   by npi;

   ffs_outpt_only = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt = 0);
   ffs_res_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt = 0);
   ffs_inpt_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt >= 1);
   ffs_outpt_res = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt = 0);
   ffs_outpt_inpt = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt >= 1);
   ffs_res_inpt = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt >= 1);
   ffs_outpt_res_inpt = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt >= 1);
   ffs_notincl_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt = 0 and
                       ffs_clmcnt_elswhr >= 1);
   ffs_partd_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt = 0 and
                     ffs_clmcnt_elswhr = 0 and ffs_rxcnt_partd >= 1);

run;

/* From the stacked Carrier, FQHC, RHC, Type II CAH, and Part D claims, roll-up to the 
   NPI-beneficiary level creating indicators that will be used to count at the NPI level
   the number of beneficiaries with dementia, the number of beneficiaries the NPI diagnosed 
   with dementia, and the number of beneficiaries by race, dual eligiblity, and LIS status. */

proc sort data=all_ffs_claims_&yrtyp.;
   by npi bene_id descending dmnt;
run;

data npi_bene_info_&yrtyp.(keep=npi ffs_num_:);
   set all_ffs_claims_&yrtyp.(keep=npi bene_id dmnt black hispanic white alloths any_lis
                                   dual dual_or_lis);
   by npi bene_id;

   if first.bene_id then do;
      ffs_num_pts = 1;
      ffs_num_pts_dement_dx = dmnt;
      ffs_num_pts_nodement_dx = not dmnt;
      ffs_num_pts_blck = black;
      ffs_num_pts_hspnc = hispanic;
      ffs_num_pts_wht = white;
      ffs_num_pts_alloths = alloths;
      ffs_num_pts_lis = any_lis;
      ffs_num_pts_nolis = not any_lis;
      ffs_num_pts_dual = dual;
      ffs_num_pts_dualorlis = dual_or_lis;
      output;
   end;

run;

/* Roll up to the NPI level counting the number and percent of beneficiaries with dementia, 
   the number and percent of beneficiaries the NPI diagnosed with dementia, and the number and
   percent of beneficiaries by race, dual eligiblity, and LIS status. */

proc means data=npi_bene_info_&yrtyp. noprint;
   by npi;
   var ffs_num_:;
   output out=npi_bene_cnts_&yrtyp.(drop=_type_ _freq_) sum=
          mean(ffs_num_pts_dement_dx ffs_num_pts_blck ffs_num_pts_hspnc
               ffs_num_pts_wht ffs_num_pts_alloths ffs_num_pts_lis
               ffs_num_pts_dual ffs_num_pts_dualorlis)=
          ffs_pct_dement_dx ffs_pct_blck ffs_pct_hspnc
          ffs_pct_wht ffs_pct_alloths ffs_pct_lis
          ffs_pct_dual ffs_pct_dualorlis;
run;

/* Merge the NPI level file with the setting indicators and zip codes and output
   a permanent file. */

data &out_pfx.&out_sfx.;
   merge npi_ffs_clm_flgs_&yrtyp.(in=a) npi_bene_cnts_&yrtyp.(in=b);
   by npi;
run;

proc datasets library=WORK kill nolist;
run;

%end;

%mend process;

%process;