/****************************************************************************************************************
   Program: 250_find_npis_allpts.sas

   Purpose: Identify the settings where NPIs provided care to all their patients (i.e., using claims for all 
			beneficiaries and not just those for patients with dementia). This code uses place of service on carrier 
			claims and revenue center codes on FQHC, RHC, and Type II CAH claims to determine settings.

   Input files: ndws_op_spec3_ffs_clms_ry.sas7bdat
                RIF(Q)yyyy.BCARRIER_CLAIMS_mm
                RIF(Q)yyyy.BCARRIER_LINE_mm

   Output files: ndws_cc_ffs_npis_allpts_ry.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

%let prognum = 250;

libname in_s3 "<input directory>" access=readonly;
libname out "<output directory>";

%let tmplib = "<temporary output directory>";

%let base_yr = 2021;
%let rcnt_yr_ey = 2023;
%let rcnt_yr_em = 9;
%let ltst_pde = 2022;

%let rifq_sy = 2023;
%let rifq_sm = 1;

%let in_s3_pfx = in_s3.ndws_op_spec3_ffs_clms_;
%let out_pfx = out.ndws_cc_ffs_npis_allpts_;

/*************************************************************************************/
/* Identify place(s) of service for all NPIs in the Carrier and FQHC/RHC/CAH II claims
/*************************************************************************************/

%macro process;

%do i = 2 %to 2;

%if &i. = 1 %then %do;
   %let start_yr = &base_yr.;
   %let start_mon = 1;
   %let yrtyp = by;
   %let s3_sfx = &yrtyp.;
   %let out_sfx = &yrtyp.;
%end;
%else %do;
   %if &rcnt_yr_em. = 12 %then %do;
      %let start_yr = &rcnt_yr_ey.;
      %let start_mon = 1;
   %end;
   %else %do;
      %let start_yr = %sysevalf(&rcnt_yr_ey. - 1);
      %let start_mon = %sysevalf(&rcnt_yr_em. + 1);
   %end;
   %let yrtyp = ry;
   %let s3_sfx = &yrtyp.;
   %let out_sfx = &yrtyp.;
%end;

%let totcnt = 12;

/* Set number of tasks (i.e. months) that will be compiled - here it will be equal to the totcnt (see above) */
%let task_compile=;
%do icnt = 1 %to &totcnt.;
   %let task_compile=&task_compile task&icnt.;
%end;

%put "***********Total number of gridding tasks - &totcnt.***********";
%put "***********List of gridding tasks - &task_compile.***********";

/* Grid Code: Grid Credentials */
OPTIONS METAUSER="&OUSER" METAPASS="&OPASSP";

/* Grid code: Enable Grid */
%LET CHG_QUEUE=QUEUE=normal; /*"normal" must be lower case*/
%LET RC=%SYSFUNC(GRDSVC_ENABLE(_ALL_, RESOURCE=sasCCW%str(;) JOBOPTS=CHG_QUEUE));

/* Loop through each task (i.e., month) */
%do cnt = 1 %to &totcnt.;

%let mon_init = %sysevalf(&start_mon. + &cnt. - 1);

%if %sysevalf(&mon_init. > 12,boolean) %then %do;
   %let mon = %sysevalf(&mon_init. - 12);
   %let yr = %sysevalf(&start_yr. + 1);
%end;
%else %do;
   %let mon = &mon_init.;
   %let yr = &start_yr;
%end;

%let mm = %sysfunc(putn(&mon., z2.));

%if &yr. > &rifq_sy. %then %do;
   %let riflib = RIFQ&yr.;
%end;
%else %if &yr. = &rifq_sy. %then %do;
   %if %sysevalf(&mon. >= &rifq_sm.,boolean) %then %do;
      %let riflib = RIFQ&yr.;
   %end;
   %else %do;
      %let riflib = RIF&yr.;
   %end;
%end;
%else %do;
   %let riflib = RIF&yr.;
%end;

/* Grid Code: Sign on to remote session for a task */
SIGNON TASK&cnt;

%syslput _all_;

%put ----> SUBMITTING: TASK&cnt. FOR YEAR &yr. MONTH &mm.;

/* Grid Code: Start remote submission for a task */
RSUBMIT TASK&cnt
CONNECTWAIT=NO /* Run tasks asynchronously */
CONNECTPERSIST=NO; /* Task end (i.e. sign off when complete) */

/*************************************************************************************/
/* Read in all Carrier lines, keeping performing NPI, place of service and physician  */
/* zip code from the first line in the claim. Identify outpatient, residential,      */
/* inpatient and other places of service.                                            */
/*************************************************************************************/

data carrier_lines_m&cnt.(keep=clm_id line_num npi line_pos physn_zip_cd);
   set &riflib..BCARRIER_LINE_&mm.(keep=clm_id line_num prf_physn_npi line_place_of_srvc_cd physn_zip_cd
                                   rename=(prf_physn_npi = npi
                                           line_place_of_srvc_cd = line_pos
                                           physn_zip_cd = zip_orig));

   length physn_zip_cd $ 9;

   physn_zip_cd = strip(zip_orig);

   if not missing(npi) then output;

run;

proc sort data=carrier_lines_m&cnt.;
   by npi clm_id line_num;
run;

data &tmplib..ndws_&prognum._carrier_claims_&yrtyp._m&cnt.(keep=npi pos_: physn_zip_cd);
   set carrier_lines_m&cnt.;
   by npi clm_id;

   if first.clm_id then do;
      pos_outpt = 0;
      pos_res = 0;
      pos_inpt = 0;
      pos_elswhr = 0;

      if line_pos in ("05","07","10","11","17","19","20","22","49","50","62","71","72") then pos_outpt = 1;
      else if line_pos in ("13","14","31","32","33","34","54") then pos_res = 1;
      else if line_pos in ("06","08","21","61") then pos_inpt = 1;
      else pos_elswhr = 1;

      output;
   end;

run;

/* Grid code: End remote submission for a task */
ENDRSUBMIT;

%put TASK&cnt CODE RAN ON MACHINE %sysfunc(GRDSVC_GETNAME(TASK&cnt));

%end; /* End task loop */

/* Grid code: Wait for all grid tasks to complete and sign off */
WAITFOR _ALL_ &task_compile;
SIGNOFF _ALL_;

/**********************************************************************************/
/* Combine all months of Carrier claims                                           */
/**********************************************************************************/

data carrier_claims_&yrtyp.;
   set &tmplib..ndws_&prognum._carrier_claims_&yrtyp._m:;
   by npi;
run;

proc datasets lib=&tmplib. nolist;
   delete ndws_&prognum._carrier_claims_&yrtyp._m:;
run;

/**********************************************************************************/
/* Read in the FQHC, RHC, Type II CAH claims from the 100 program and rename the  */
/* setting variables so we can stack with Carrier claims pulled above. Set the    */
/* inpatient setting to 0. Assign an NPI to the claim as the attending NPI if not */
/* missing, otherwise use the rendering NPI or organizational NPI. Note, the      */
/* attending NPI was almost always non-missing.                                   */
/**********************************************************************************/

data fqhcrhccah_claims_&yrtyp.(keep=npi pos_: physn_zip_cd);
   set &in_s3_pfx.&s3_sfx.(keep=snf_visit homeoth_visit office_visit
                                at_physn_npi rndrng_physn_npi org_npi_num clm_srvc_fac_zip_cd
                           rename=(snf_visit = pos_res
                                   homeoth_visit = pos_elswhr
                                   office_visit = pos_outpt
                                   clm_srvc_fac_zip_cd = physn_zip_cd));

   length npi $ 12;

   if missing(at_physn_npi) then do;
      if missing(rndrng_physn_npi) then npi = strip(org_npi_num);
      else npi = strip(rndrng_physn_npi);
   end;
   else npi = strip(at_physn_npi);

   pos_inpt = 0;

   if not missing(npi) then output;

run;

proc sort data=fqhcrhccah_claims_&yrtyp.;
   by npi;
run;

/**********************************************************************************/
/* Stack Carrier and FQHC/RHC/Type II CAH claims and roll up to the NPI level     */
/* counting the number of claims billed to each setting and identifing the zip    */
/* code that the NPI billed to most often among all their patients.               */
/**********************************************************************************/

data all_ffs_claims_&yrtyp.(drop=pos_:);
   set carrier_claims_&yrtyp.(in=a)
       fqhcrhccah_claims_&yrtyp.(in=b);
   by npi;

   carrier = a;
   fqhcrhccah = b;

   num_zip_claims = 1;

   /* Set claim level flags for counting claims by place of service in the step below */
   ffs_clmcnt_res = pos_res;
   ffs_clmcnt_inpt = pos_inpt;
   ffs_clmcnt_outpt = pos_outpt;
   ffs_clmcnt_elswhr = pos_elswhr;

run;

/* Count the number of claims by setting for each NPI */
proc means data=all_ffs_claims_&yrtyp. noprint;
   by npi;
   var ffs_:;
   output out=npi_ffs_clm_cnts_&yrtyp.(drop=_type_ _freq_) sum=;
run;

/* Count the number of claims billed to each practice zip code by NPI */
proc means data=all_ffs_claims_&yrtyp. noprint;
   class npi physn_zip_cd;
   types npi*physn_zip_cd;
   var num_zip_claims;
   output out=ptb_zip_claim_cnts_&yrtyp.(drop=_type_ _freq_) sum=;
run;

proc sort data=ptb_zip_claim_cnts_&yrtyp.;
   by npi descending num_zip_claims;
run;

/* Identify the practice zip code the NPI billed to most often */
data npi_ptb_zips_&yrtyp.(keep=npi physn_zip_cd rename=(physn_zip_cd = allpts_zip1));
   set ptb_zip_claim_cnts_&yrtyp.;
   by npi;
   if first.npi then output;
run;

/* Merge the top zip code to the NPI level file with claim counts by setting 
   and assign the setting variables, and a categorical variable for setting
   based on all patients the NPI treated. */

data &out_pfx.&out_sfx.(keep=npi allpts_:);
   merge npi_ffs_clm_cnts_&yrtyp. npi_ptb_zips_&yrtyp.;
   by npi;

   ffs_outpt_only = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt = 0);
   ffs_res_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt = 0);
   ffs_inpt_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt >= 1);
   ffs_outpt_res = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt = 0);
   ffs_outpt_inpt = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt >= 1);
   ffs_res_inpt = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt >= 1);
   ffs_outpt_res_inpt = (ffs_clmcnt_outpt >= 1 and ffs_clmcnt_res >= 1 and ffs_clmcnt_inpt >= 1);
   ffs_notincl_only = (ffs_clmcnt_outpt = 0 and ffs_clmcnt_res = 0 and ffs_clmcnt_inpt = 0 and
                       ffs_clmcnt_elswhr >= 1);

   allpts_ffs_setting = 0;

   if ffs_outpt_only then allpts_ffs_setting = 1;
   else if ffs_res_only then allpts_ffs_setting = 2;
   else if ffs_inpt_only then allpts_ffs_setting = 3;
   else if ffs_outpt_res then allpts_ffs_setting = 4;
   else if ffs_outpt_inpt then allpts_ffs_setting = 5;
   else if ffs_res_inpt then allpts_ffs_setting = 6;
   else if ffs_outpt_res_inpt then allpts_ffs_setting = 7;
   else if ffs_notincl_only then allpts_ffs_setting = 9;

run;

proc datasets library=WORK kill nolist;
run;

%end;

%mend process;

%process;