/****************************************************************************************************************
   Program: 310_find_ma_npis.sas

   Purpose: Find all the clinicians who treated MA beneficiaries with dementia for CY 2021. This 
            program pulls MA encounter data for beneficiaries identified as having dementia
            in MA encounter data.

   Input files: ndws_cc_bene_ff_mbsf_by.sas7bdat
                _101_op_dmnt_flgd_frc_2021.sas7bdat
                _101_car_dmnt_flgd_2021.sas7bdat
                PDE2021.PDE_ENCRYPT_LINK_2021_mm

   Output files: ndws_cc_ma_npis_by.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

%let prognum = 310;

libname in_ff "<finder file directory>" access=readonly;
libname in_clm "<ma directory>" access=readonly;
libname out "<out directory>";

%let tmplib = "<temporary output directory>";

%let yr = 2021;

%let in_ff = in_ff.ndws_cc_bene_ff_mbsf_by;
%let in_opt = in_clm._101_op_dmnt_flgd_frc_&yr.;
%let in_car = in_clm._101_car_dmnt_flgd_&yr.;
%let outfile = out.ndws_cc_ma_npis_by;

/*****************************************************************************/
/* Identify NPIs who treated beneficiaries with dementia in MA encounter data */
/*****************************************************************************/

%macro process;

%let totcnt = 12;

/* Set number of tasks (i.e., months) that will be compiled - here it will be equal to the totcnt (see above) */
%let task_compile=;
%do icnt = 1 %to &totcnt.;
   %let task_compile=&task_compile task&icnt.;
%end;

%put "***********Total number of gridding tasks - &totcnt.***********";
%put "***********List of gridding tasks - &task_compile.***********";

/* Grid Code: Grid Credentials */
OPTIONS METAUSER="&OUSER" METAPASS="&OPASSP";

/* Grid code: Enable Grid */
%LET CHG_QUEUE=QUEUE=normal; /*"normal" must be lower case*/
%LET RC=%SYSFUNC(GRDSVC_ENABLE(_ALL_, RESOURCE=sasCCW%str(;) JOBOPTS=CHG_QUEUE));

/* Loop through each task (i.e., month) */
%do cnt = 1 %to &totcnt.;

%let mm = %sysfunc(putn(&cnt., z2.));

/* Grid Code: Sign on to remote session for a task */
SIGNON TASK&cnt;

%syslput _all_;

%put ----> SUBMITTING: TASK&cnt. FOR YEAR &yr. MONTH &mm.;

/* Grid Code: Start remote submission for a task */
RSUBMIT TASK&cnt
CONNECTWAIT=NO /* Run tasks asynchronously */
CONNECTPERSIST=NO; /* Task end (i.e. sign off when complete) */

libname in_ff "<finder file directory>" access=readonly;
libname in_clm "<ma directory>" access=readonly;

/**********************************************************************************/
/* Step 1: Process Carrier Encounter Data                                         */
/*                                                                                */
/* Pull carrier encounter records (from the 101 program, which already contain    */
/* flags for whether the record has a dementia diagnosis). Limit to final action, */
/* non chart review records and to records for MA beneficiaries with dementia in  */
/* the finder file. In this program, we use header-level place of service,        */
/* header-level rendering NPI when not missing, and otherwise the organizational  */
/* NPI due to high rates of missingness in place or service and NPI fields in     */
/* the line level data.														      */ 
/*                                                                                */
/* Create flags for the settings where NPIs saw beneficiaries with dementia       */
/* (outpatient, residential, inpatient, other, or Part D prescriber (set to zero  */
/* at this stage) 																  */
/*                                                                                */
/* Beneficiary eligibility and demographic information from the finder file is    */
/* merged onto each encounter record.                                             */
/**********************************************************************************/

data &tmplib..ndws_&prognum._carrier_claims_m&cnt.(keep=bene_id dmnt
                                                        black hispanic white alloths any_lis dual dual_or_lis
                                                        npi pos_:);
   merge &in_ff.(in=a keep=bene_id ma_enctr black hispanic white alloths any_lis dual dual_or_lis
                      where=(ma_enctr))
         &in_car.(in=b keep=bene_id clm_thru_dt clm_finl_actn_ind dement clm_chrt_rvw_sw
                            rndrng_physn_npi org_npi clm_place_of_srvc_cd
                       where=(month(clm_thru_dt) = &cnt. and clm_finl_actn_ind = "Y" and
                              not (clm_chrt_rvw_sw = "Y"))
                       rename=(dement = dmnt
                               clm_place_of_srvc_cd = clm_pos));
   by bene_id;
   if a and b;

   length npi $ 10;

   if missing(rndrng_physn_npi) then npi = org_npi;
   else npi = rndrng_physn_npi;

   pos_outpt = 0;
   pos_res = 0;
   pos_inpt = 0;
   pos_elswhr = 0;
   pos_partd = 0;

   if clm_pos in ("05","07","10","11","17","19","20","22","49","50","62","71","72") then pos_outpt = 1;
   else if clm_pos in ("13","14","31","32","33","34","54") then pos_res = 1;
   else if clm_pos in ("06","08","21","61") then pos_inpt = 1;
   else pos_elswhr = 1;

run;

/**********************************************************************************/
/* Step 2: Process FQHC, RHC, Type II CAH Encounter Data                          */
/*                                                                                */
/* From the FQHC, RHC, Type II CAH encounter records with dementia diagnoses      */
/* created in the 101 program, limit to final action, non chart review records    */
/* and to records for MA beneficiaries with dementia in the finder file. Rename   */
/* the setting variables so we can stack with Carrier data above and set the      */
/* inpatient and Part D prescriber settings to zero. If the attending NPI (renamed*/
/* to NPI in the 101 program) is missing, use the rendering NPI if not missing,   */
/* else the organizational NPI.                                                   */
/*                                                                                */
/* Keep the dementia flag created in the 101 program, NPI, setting indicators,    */
/* and the beneficiary eligibility and demographic information from the finder    */
/* file.                                                                          */
/**********************************************************************************/

data &tmplib..ndws_&prognum._frc_claims_m&cnt.(keep=bene_id dmnt
                                                    black hispanic white alloths any_lis dual dual_or_lis
                                                    npi pos_:);
   merge &in_ff.(in=a keep=bene_id ma_enctr black hispanic white alloths any_lis dual dual_or_lis
                      where=(ma_enctr))
         &in_opt.(in=b keep=bene_id clm_thru_dt clm_finl_actn_ind dement
                            clm_chrt_rvw_sw snf_visit homeoth_visit office_visit
                            npi rndrng_physn_npi org_npi
                       where=(month(clm_thru_dt) = &cnt. and clm_finl_actn_ind = "Y" and
                              not (clm_chrt_rvw_sw = "Y"))
                       rename=(dement = dmnt
                               snf_visit = pos_res
                               homeoth_visit = pos_elswhr
                               office_visit = pos_outpt));
   by bene_id;
   if a and b;

   if missing(npi) then do;
      if missing(rndrng_physn_npi) then npi = org_npi;
      else npi = rndrng_physn_npi;
   end;

   pos_inpt = 0;
   pos_partd = 0;

run;

/**********************************************************************************/
/* Step 3: Process Part D Drug Event Data                                         */
/*                                                                                */
/* Pull Part D claims for MA beneficiaries with dementia to identify and create   */
/* the same setting flags as carrier (outpatient, residential, inpatient, other,  */
/* and Part D prescriber). All settings except Part D prescriber are set to zero. */
/*                                                                                */
/* Beneficiary eligibility and demographic information from the finder file is    */
/* merged onto each Part D claim.                                                 */
/**********************************************************************************/

data &tmplib..ndws_&prognum._ptd_claims_m&cnt.(keep=pos_: npi
                                                    bene_id black hispanic white alloths any_lis dual dual_or_lis);
   merge &in_ff.(in=a keep=bene_id ma_enctr black hispanic white alloths any_lis dual dual_or_lis
                      where=(ma_enctr))
         PDE&yr..PDE_ENCRYPT_LINK_&yr._&mm.(in=b keep=bene_id prscrbr_id prscrbr_id_qlfyr_cd
                                                 where=(not (missing(prscrbr_id) or
                                                             missing(prscrbr_id_qlfyr_cd) or
                                                             prscrbr_id_qlfyr_cd in ("06","08","12"))));
   by bene_id;
   if a and b;

   length npi $ 10;

   npi = strip(prscrbr_id);

   pos_outpt = 0;
   pos_res = 0;
   pos_inpt = 0;
   pos_elswhr = 0;
   pos_partd = 1;

run;

/* Grid code: End remote submission for a task */
ENDRSUBMIT;

%put TASK&cnt CODE RAN ON MACHINE %sysfunc(GRDSVC_GETNAME(TASK&cnt));

%end; /* End task loop */

/* Grid code: Wait for all grid tasks to complete and sign off */
WAITFOR _ALL_ &task_compile;
SIGNOFF _ALL_;

/**********************************************************************************/
/* Combine all months of Carrier records pulled for beneficiaries with dementia   */
/**********************************************************************************/

data carrier_claims;
   set &tmplib..ndws_&prognum._carrier_claims_m:;
   by bene_id;
run;

/**********************************************************************************/
/* Combine all months of FQHC, RHC, Type II CAH records pulled for beneficiaries  */
/* with dementia                                                                  */
/**********************************************************************************/

data frc_claims;
   set &tmplib..ndws_&prognum._frc_claims_m:;
   by bene_id;
run;

/**********************************************************************************/
/* Combine all months of Part D event data pulled for beneficiaries with dementia */
/**********************************************************************************/

data ptd_claims;
   set &tmplib..ndws_&prognum._ptd_claims_m:;
run;

proc datasets lib=&tmplib. nolist;
   delete ndws_&prognum._carrier_claims_m:
          ndws_&prognum._frc_claims_m:
          ndws_&prognum._ptd_claims_m:;
run;

/**********************************************************************************/
/* Step 4: Stack Carrier, Part D, FQHC/RHC/Type II CAH records and roll up to the */
/* NPI level.                                                                     */
/*                                                                                */
/* The steps below will roll-up the Carrier, Part D, FQHC/RHC/Type II CAH records */
/* for beneficiaries with dementia to the NPI level, creating variables for the   */
/* settings where each NPI provided care to their dementia patients, and counts of*/
/* the number of beneficiaries with dementia served (total and number dually      */
/* eligible or eligible for a low-income drug drug subsidy), and the number of    */
/* beneficiaries who each NPI diagnosed with dementia.                            */
/**********************************************************************************/

data all_ma_claims(drop=pos_:);
   set carrier_claims(in=a)
       frc_claims(in=b)
       ptd_claims(in=c);

   carrier = a;
   fqhcrhccah = b;
   partd = c;

   if c then dmnt = 0;

   /* Set encounter level flags for counting encounters by place of service in the step below */
   ma_clmcnt_res = pos_res;
   ma_clmcnt_inpt = pos_inpt;
   ma_clmcnt_outpt = pos_outpt;
   ma_clmcnt_elswhr = pos_elswhr;
   ma_rxcnt_partd = pos_partd;

   /* These flags will be used to count encounters from the Carrier file by place of 
      service in the step below */
   ma_ptb_pos_res = carrier and pos_res;
   ma_ptb_pos_inpt = carrier and pos_inpt;
   ma_ptb_pos_outpt = carrier and pos_outpt;
   ma_ptb_pos_elswhr = carrier and pos_elswhr;

   if not missing(npi) then output;

run;

/* Count the number of encounters by setting for each NPI */
proc means data=all_ma_claims noprint;
   class npi;
   types npi;
   var ma_:;
   output out=npi_ma_clm_cnts(drop=_type_ _freq_) sum=;
run;

/* Assign the setting variables */
data npi_ma_clm_flgs;
   set npi_ma_clm_cnts;
   by npi;

   ma_outpt_only = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res = 0 and ma_clmcnt_inpt = 0);
   ma_res_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt = 0);
   ma_inpt_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res = 0 and ma_clmcnt_inpt >= 1);
   ma_outpt_res = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt = 0);
   ma_outpt_inpt = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res = 0 and ma_clmcnt_inpt >= 1);
   ma_res_inpt = (ma_clmcnt_outpt = 0 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt >= 1);
   ma_outpt_res_inpt = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt >= 1);
   ma_notincl_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res = 0 and ma_clmcnt_inpt = 0 and
                      ma_clmcnt_elswhr >= 1);
   ma_partd_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res = 0 and ma_clmcnt_inpt = 0 and
                    ma_clmcnt_elswhr = 0 and ma_rxcnt_partd >= 1);

run;

/* From the stacked Carrier, FQHC, RHC, Type II CAH, and Part D claims, roll up to the 
   NPI-beneficiary level creating indicators that will be used to count at the NPI level
   the number of beneficiaries with dementia, the number of beneficiaries the NPI diagnosed 
   with dementia, and the number of beneficiaries by race, dual eligiblity, and LIS status. */

proc sort data=all_ma_claims;
   by npi bene_id descending dmnt;
run;

data npi_bene_info(keep=npi ma_num_:);
   set all_ma_claims(keep=npi bene_id dmnt black hispanic white alloths any_lis dual dual_or_lis);
   by npi bene_id;

   if first.bene_id then do;
      ma_num_pts = 1;
      ma_num_pts_dement_dx = dmnt;
      ma_num_pts_nodement_dx = not dmnt;
      ma_num_pts_blck = black;
      ma_num_pts_hspnc = hispanic;
      ma_num_pts_wht = white;
      ma_num_pts_alloths = alloths;
      ma_num_pts_lis = any_lis;
      ma_num_pts_nolis = not any_lis;
      ma_num_pts_dual = dual;
      ma_num_pts_dualorlis = dual_or_lis;
      output;
   end;

run;

/* Roll up to the NPI level counting the number and percent of beneficiaries with dementia, 
   the number and percent of beneficiaries the NPI diagnosed with dementia, and the number and
   percent of beneficiaries by race, dual eligiblity, and LIS status. */

proc means data=npi_bene_info noprint;
   by npi;
   var ma_num_:;
   output out=npi_bene_cnts(drop=_type_ _freq_) sum=
          mean(ma_num_pts_dement_dx ma_num_pts_blck ma_num_pts_hspnc
               ma_num_pts_wht ma_num_pts_alloths ma_num_pts_lis
               ma_num_pts_dual ma_num_pts_dualorlis)=
          ma_pct_dement_dx ma_pct_blck ma_pct_hspnc
          ma_pct_wht ma_pct_alloths ma_pct_lis
          ma_pct_dual ma_pct_dualorlis;
run;

/* Merge the NPI level file with the setting indicators and output
   a permanent file. */

data &outfile.;
   merge npi_ma_clm_flgs npi_bene_cnts;
   by npi;
run;

proc datasets library=WORK kill nolist;
run;

%mend process;

%process;