/****************************************************************************************************************
   Program: 350_find_ma_npis_allpts.sas

   Purpose: Identify the settings where NPIs provided care to all their patients (i.e., using encounter records
			for all beneficiaries and not just those with dementia). This code uses place of serivce on carrier
			encounter records and revenue center codes on encounter records from FQHCs, RHCs, and type II CAHs
			to determine settings. 

   Input files: _101_op_dmnt_flgd_frc_2021.sas7bdat
                _101_car_dmnt_flgd_2021.sas7bdat

   Output files: ndws_cc_ma_npis_allpts_by.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

%let prognum = 350;

libname in_clm "<ma input directory>" access=readonly;
libname out "<output directory>";

%let tmplib = "<temporary output directory>";

%let yr = 2021;

%let in_opt = in_clm._101_op_dmnt_flgd_frc_&yr.;
%let in_car = in_clm._101_car_dmnt_flgd_&yr.;
%let outfile = out.ndws_cc_ma_npis_allpts_by;

/*********************************************************************************************/
/* Identify place(s) of service for all NPI in the Carrier and FQHC/RHC/CAH II encounter data
/*********************************************************************************************/

%macro process;

%let totcnt = 12;

/* Set number of tasks (i.e., months) that will be compiled - here it will be equal to the totcnt (see above) */
%let task_compile=;
%do icnt = 1 %to &totcnt.;
   %let task_compile=&task_compile task&icnt.;
%end;

%put "***********Total number of gridding tasks - &totcnt.***********";
%put "***********List of gridding tasks - &task_compile.***********";

/* Grid Code: Grid Credentials */
OPTIONS METAUSER="&OUSER" METAPASS="&OPASSP";

/* Grid code: Enable Grid */
%LET CHG_QUEUE=QUEUE=normal; /*"normal" must be lower case*/
%LET RC=%SYSFUNC(GRDSVC_ENABLE(_ALL_, RESOURCE=sasCCW%str(;) JOBOPTS=CHG_QUEUE));

/* Loop through each task (i.e., month) */
%do cnt = 1 %to &totcnt.;

%let mm = %sysfunc(putn(&cnt., z2.));

/* Grid Code: Sign on to remote session for a task */
SIGNON TASK&cnt;

%syslput _all_;

%put ----> SUBMITTING: TASK&cnt. FOR YEAR &yr. MONTH &mm.;

/* Grid Code: Start remote submission for a task */
RSUBMIT TASK&cnt
CONNECTWAIT=NO /* Run tasks asynchronously */
CONNECTPERSIST=NO; /* Task end (i.e. sign off when complete) */

libname in_clm "<ma input directory>" access=readonly;

/**********************************************************************************/
/* Step 1: Process Carrier Encounter Data                                         */
/*                                                                                */
/* Pull carrier encounter records (from the 101 program). Limit to final action,  */
/* non chart review records. In this program, we use header-level place of        */
/* service, header-level rendering NPI when not missing, and otherwise the        */
/* organizational NPI due to high rates of missingness in place or service and    */
/* NPI fields in the line level data.										      */ 
/*                                                                                */
/* Create flags for the settings where NPIs saw beneficiaries with dementia       */
/* (outpatient, residential, inpatient, or other)								  */
/**********************************************************************************/
data &tmplib..ndws_&prognum._carrier_claims_m&cnt.(keep=npi pos_:);
   set &in_car.(keep=clm_thru_dt clm_finl_actn_ind clm_chrt_rvw_sw
                     rndrng_physn_npi org_npi clm_place_of_srvc_cd
                where=(month(clm_thru_dt) = &cnt. and clm_finl_actn_ind = "Y" and
                       not (clm_chrt_rvw_sw = "Y"))
                rename=(clm_place_of_srvc_cd = clm_pos));

   length npi $ 10;

   if missing(rndrng_physn_npi) then npi = org_npi;
   else npi = rndrng_physn_npi;

   pos_outpt = 0;
   pos_res = 0;
   pos_inpt = 0;
   pos_elswhr = 0;

   if clm_pos in ("05","07","10","11","17","19","20","22","49","50","62","71","72") then pos_outpt = 1;
   else if clm_pos in ("13","14","31","32","33","34","54") then pos_res = 1;
   else if clm_pos in ("06","08","21","61") then pos_inpt = 1;
   else pos_elswhr = 1;

   if not missing(npi) then output;

run;

/**********************************************************************************/
/* Step 2: Process FQHC, RHC, Type II CAH Encounter Data                          */
/*                                                                                */
/* From the FQHC, RHC, Type II CAH encounter records with dementia claims flagged */
/* created in program 101, limit to final action, non chart review records.       */
/* Rename the setting variables so we can stack with Carrier data pulled above    */
/* and set the inpatient setting to zero. If the attending NPI (renamed to NPI in */
/* the 101 program) is missing, use the rendering NPI if not missing, else the    */
/* organizational NPI.                                                            */
/*                                                                                */
/* Keep the NPI and place of service indicators.                                  */
/**********************************************************************************/

data &tmplib..ndws_&prognum._frc_claims_m&cnt.(keep=npi pos_:);
   set &in_opt.(keep=clm_thru_dt clm_finl_actn_ind
                     clm_chrt_rvw_sw snf_visit homeoth_visit office_visit
                     npi rndrng_physn_npi org_npi
                where=(month(clm_thru_dt) = &cnt. and clm_finl_actn_ind = "Y" and
                       not (clm_chrt_rvw_sw = "Y"))
                rename=(snf_visit = pos_res
                        homeoth_visit = pos_elswhr
                        office_visit = pos_outpt));

   if missing(npi) then do;
      if missing(rndrng_physn_npi) then npi = org_npi;
      else npi = rndrng_physn_npi;
   end;

   pos_inpt = 0;

   if not missing(npi) then output;

run;

/* Grid code: End remote submission for a task */
ENDRSUBMIT;

%put TASK&cnt CODE RAN ON MACHINE %sysfunc(GRDSVC_GETNAME(TASK&cnt));

%end; /* End task loop */

/* Grid code: Wait for all grid tasks to complete and sign off */
WAITFOR _ALL_ &task_compile;
SIGNOFF _ALL_;

/**********************************************************************************/
/* Combine all months of Carrier records                                          */
/**********************************************************************************/

data carrier_claims;
   set &tmplib..ndws_&prognum._carrier_claims_m:;
run;

/**********************************************************************************/
/* Combine all months of FQHC, RHC, Type II CAH records                           */
/**********************************************************************************/

data frc_claims;
   set &tmplib..ndws_&prognum._frc_claims_m:;
run;

proc datasets lib=&tmplib. nolist;
   delete ndws_&prognum._carrier_claims_m:
          ndws_&prognum._frc_claims_m:;
run;

/**********************************************************************************/
/* Step 3: Stack Carrier and FQHC/RHC/Type II CAH records and roll up to the      */
/* NPI level.                                                                     */
/*                                                                                */
/* The steps below will roll-up the Carrier and FQHC/RHC/Type II CAH records      */
/* to the NPI level, creating variables for the settings where each NPI provided  */
/* care to their patients and counts of the number of beneficiairies served       */
/**********************************************************************************/

data all_ma_claims(drop=pos_:);
   set carrier_claims
       frc_claims;

   /* Set encounter level flags for counting encounters by place of service in the step below */
   ma_clmcnt_res = pos_res;
   ma_clmcnt_inpt = pos_inpt;
   ma_clmcnt_outpt = pos_outpt;
   ma_clmcnt_elswhr = pos_elswhr;

run;

/* Count the number of encounters by setting for each NPI */
proc means data=all_ma_claims noprint;
   class npi;
   types npi;
   var ma_:;
   output out=npi_ma_clm_cnts(drop=_type_ _freq_) sum=;
run;

/* Assign the setting variables and output the file */
data &outfile.(keep=npi allpts_:);
   set npi_ma_clm_cnts;

   ma_outpt_only = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res = 0 and ma_clmcnt_inpt = 0);
   ma_res_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt = 0);
   ma_inpt_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res = 0 and ma_clmcnt_inpt >= 1);
   ma_outpt_res = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt = 0);
   ma_outpt_inpt = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res = 0 and ma_clmcnt_inpt >= 1);
   ma_res_inpt = (ma_clmcnt_outpt = 0 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt >= 1);
   ma_outpt_res_inpt = (ma_clmcnt_outpt >= 1 and ma_clmcnt_res >= 1 and ma_clmcnt_inpt >= 1);
   ma_notincl_only = (ma_clmcnt_outpt = 0 and ma_clmcnt_res = 0 and ma_clmcnt_inpt = 0 and
                      ma_clmcnt_elswhr >= 1);

   allpts_ma_setting = 0;

   if ma_outpt_only then allpts_ma_setting = 1;
   else if ma_res_only then allpts_ma_setting = 2;
   else if ma_inpt_only then allpts_ma_setting = 3;
   else if ma_outpt_res then allpts_ma_setting = 4;
   else if ma_outpt_inpt then allpts_ma_setting = 5;
   else if ma_res_inpt then allpts_ma_setting = 6;
   else if ma_outpt_res_inpt then allpts_ma_setting = 7;
   else if ma_notincl_only then allpts_ma_setting = 9;

run;

proc datasets library=WORK kill nolist;
run;

%mend process;

%process;