/****************************************************************************************************************
   Program: 410_merge_npis.sas

   Purpose: Merge data for clinicians who treated FFS and MA beneficiaries with dementia in CY 2021 to calculate 
			the total number of beneficiaries with dementia served across FFS and MA and create categorical setting 
			variables.

   Input files: ndws_cc_ffs_npis_by.sas7bdat
                ndws_cc_ma_npis_by.sas7bdat

   Output files: ndws_cc_all_npis_by.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;

%let prognum = 410;

libname in_ffs "<ffs directory>" access=readonly;
libname in_ma "<ma directory>" access=readonly;
libname out "<output directory>";

%let in_ffs = in_ffs.ndws_cc_ffs_npis_by;
%let in_ma = in_ma.ndws_cc_ma_npis_by;
%let outfile = out.ndws_cc_all_npis_by;

/**********************************************************************************/
/* Read in the CY 2021 FFS derived NPI-level file                                 */
/**********************************************************************************/

data ffs_npis(drop=npi_orig);
   set &in_ffs.(rename=(npi = npi_orig));

   length npi $ 10;

   npi = strip(npi_orig);

run;

/**********************************************************************************/
/* Merge the CY 2021 FFS- and CY 2021 MA-derived NPI-level files and create flags */
/* for whether the NPI is in both FFS and MA files, only FFS, or only MA. Sum the */
/* number of beneficiaries served overall and by type (e.g., dually eligible or   */
/* eligible for low-income drug subsidy) across FFS and MA and calculate the      */
/* percent of the NPI's panel that was FFS overall and by type                    */
/**********************************************************************************/

data all_npis;
   merge ffs_npis(in=a) &in_ma.(in=b);
   by npi;

   label npi = " ";

   in_ffsma = (a and b);
   in_ffsonly = (a and not b);
   in_maonly = (b and not a);

   array ffs_vars ffs_:;
   array ma_vars ma_:;

   if not a then do;
      do over ffs_vars;
         ffs_vars = 0;
      end;
   end;

   if not b then do;
      do over ma_vars;
         ma_vars = 0;
      end;
   end;

   tot_num_pts = ffs_num_pts + ma_num_pts;
   tot_num_pts_dement_dx = ffs_num_pts_dement_dx + ma_num_pts_dement_dx;
   tot_num_pts_blck = ffs_num_pts_blck + ma_num_pts_blck;
   tot_num_pts_hspnc = ffs_num_pts_hspnc + ma_num_pts_hspnc;
   tot_num_pts_wht = ffs_num_pts_wht + ma_num_pts_wht;
   tot_num_pts_alloths = ffs_num_pts_alloths + ma_num_pts_alloths;
   tot_num_pts_lis = ffs_num_pts_lis + ma_num_pts_lis;
   tot_num_pts_nolis = ffs_num_pts_nolis + ma_num_pts_nolis;
   tot_num_pts_dual = ffs_num_pts_dual + ma_num_pts_dual;
   tot_num_pts_dualorlis = ffs_num_pts_dualorlis + ma_num_pts_dualorlis;

   if tot_num_pts = 0 then npi_pct_pts_ffs = 0;
   else npi_pct_pts_ffs = ffs_num_pts / tot_num_pts;

   if tot_num_pts_blck = 0 then npi_pct_blck_ffs = 0;
   else npi_pct_blck_ffs = ffs_num_pts_blck / tot_num_pts_blck;

   if tot_num_pts_hspnc = 0 then npi_pct_hspnc_ffs = 0;
   else npi_pct_hspnc_ffs = ffs_num_pts_hspnc / tot_num_pts_hspnc;

   if tot_num_pts_wht = 0 then npi_pct_wht_ffs = 0;
   else npi_pct_wht_ffs = ffs_num_pts_wht / tot_num_pts_wht;

   if tot_num_pts_alloths = 0 then npi_pct_alloths_ffs = 0;
   else npi_pct_alloths_ffs = ffs_num_pts_alloths / tot_num_pts_alloths;

   if tot_num_pts_lis = 0 then npi_pct_lis_ffs = 0;
   else npi_pct_lis_ffs = ffs_num_pts_lis / tot_num_pts_lis;

   if tot_num_pts_dual = 0 then npi_pct_dual_ffs = 0;
   else npi_pct_dual_ffs = ffs_num_pts_dual / tot_num_pts_dual;

   if tot_num_pts_dualorlis = 0 then npi_pct_dualorlis_ffs = 0;
   else npi_pct_dualorlis_ffs = ffs_num_pts_dualorlis / tot_num_pts_dualorlis;

run;

/**********************************************************************************/
/* For both FFS and MA separately, create a 9-category setting categorical        */
/* variable and output a permanent file.                                          */
/**********************************************************************************/

data &outfile.;
   set all_npis;

   if ffs_outpt_only then ffs_setting_check = 1;
   else if ffs_res_only then ffs_setting_check = 2;
   else if ffs_inpt_only then ffs_setting_check = 3;
   else if ffs_outpt_res then ffs_setting_check = 4;
   else if ffs_outpt_inpt then ffs_setting_check = 5;
   else if ffs_res_inpt then ffs_setting_check = 6;
   else if ffs_outpt_res_inpt then ffs_setting_check = 7;
   else if ffs_partd_only then ffs_setting_check = 8;
   else if ffs_notincl_only then ffs_setting_check = 9;

   if ma_outpt_only then ma_setting_check = 1;
   else if ma_res_only then ma_setting_check = 2;
   else if ma_inpt_only then ma_setting_check = 3;
   else if ma_outpt_res then ma_setting_check = 4;
   else if ma_outpt_inpt then ma_setting_check = 5;
   else if ma_res_inpt then ma_setting_check = 6;
   else if ma_outpt_res_inpt then ma_setting_check = 7;
   else if ma_partd_only then ma_setting_check = 8;
   else if ma_notincl_only then ma_setting_check = 9;

run;