/****************************************************************************************************************
   Program: 500_create_sample_frame.sas

   Purpose: Identify the initial set of NPIs for the CC sample frame and construct variables for the sample frame file

   Input files: ndws_cc_all_npis_by.sas7bdat
                ndws_cc_all_npis_mds_only_by.sas7bdat
                ndws_cc_ffs_npis_ry.sas7bdat
                ndws_cc_ffs_npis_allpts_ry.sas7bdat
                ndws_cc_ma_npis_allpts_by.sas7bdat
                _020_npi.sas7bdat
                enrlmnt_extract.sas7bdat
                zip_zcta.sas7bdat
                zcta_cnty.sas7bdat
                cnty_geo_var_2022.sas7bdat
                cnty_geo_var_2021.sas7bdat

   Output files: ndws_cc_sample_frame.sas7bdat
                 500_all_npis_by.csv
                 500_all_npis_ry.csv
                 500_ma_only_by_ffs_ry_npis.csv
****************************************************************************************************************/
options nocenter mprint obs=max;

%let prognum = 500;

libname in_cmbd "<combined directory>" access=readonly;
libname in_ffs "<ffs directory" access=readonly;
libname in_ma "ma directory" access=readonly;
libname in_nppes "<nppes directory>" access=readonly;
libname in_pecos "<pecos directory>" access=readonly;
libname in_zcta "<zcta directory>" access=readonly;
libname in_gvdb "<gvdb directory>" access=readonly;

libname out "<out directory>";

%let in_by = in_cmbd.ndws_cc_all_npis_by;
%let in_by_mds = in_cmbd.ndws_cc_all_npis_mds_only_by;
%let in_ry = in_ffs.ndws_cc_ffs_npis_ry;
%let in_by_allpts = in_ma.ndws_cc_ma_npis_allpts_by;
%let in_ry_allpts = in_ffs.ndws_cc_ffs_npis_allpts_ry;
%let in_nppes = in_nppes._020_npi;
%let in_enrl = in_pecos.enrlmnt_extract;
%let in_zip_zcta = in_zcta.zip_zcta;
%let in_zcta_cnty = in_zcta.zcta_cnty;
%let in_cnty_var = in_gvdb.cnty_geo_var_2022;
%let in_cnty_var_alt = in_gvdb.cnty_geo_var_2021;
%let outfile = out.ndws_cc_sample_frame;

%let outnpi1 = <out directory>/&prognum._all_npis_by.csv;
%let outnpi2 = <out directory>/&prognum._all_npis_ry.csv;
%let outnpi3 = <out directory>/&prognum._ma_only_by_ffs_ry_npis.csv;

%let keepvars_num = fnl_d_pts fnl_d_blckhspnc fnl_d_dualorlis fnl_d_pts_dxed;

%let keepvars = npi
                enum_date_2024 prov_nm_pre_2024 prov_fn_2024 prov_ln_2024 prov_nm_suff_2024
                priority_address_1 priority_address_2 priority_city priority_state priority_zip
                priority_cntry taxonomy_2024 clinician_type ffs_setting_check ma_setting_check
                outofusa in2021 &keepvars_num.;

/***********************************************************************************/
/* Step 1: Merge NPPES data from the 020 program onto the NPI-level files from the */ 
/* 400 programs to identify NPIs with relevant taxononmies who are currently active*/                                          
/***********************************************************************************/
/***********************************************************************************/
/* Prepare NPPES data and assign clinician type based on specialty and licensing.  */
/* Create a 7-category variable for clinician type that describes the licensure    */
/* (MD/DO, NP, or PA) and specialty (primary care, neurology, and psychology) of   */
/* clinicians targeted for the survey. NPIs with taxonomies that are not relevant  */
/* to the survey are set to missing.  Flag taxonomy codes for clinical nurse       */
/* specialists that were initially flagged for inclusion, but subsequently         */
/* excluded. 																	   */
/***********************************************************************************/

data nppes(drop=txnmy_2024);
   set &in_nppes.(rename=(taxonomy_2024 = txnmy_2024));

   length taxonomy_2024 $ 10;

   taxonomy_2024 = strip(txnmy_2024);

   if dm_specialty_2024 = 1 and licensing_2024 = 0 then clinician_type = 1;
   else if dm_specialty_2024 = 1 and licensing_2024 = 1 then clinician_type = 2;
   else if dm_specialty_2024 = 1 and licensing_2024 = 2 then clinician_type = 3;
   else if dm_specialty_2024 = 3 and licensing_2024 = 0 then clinician_type = 4;
   else if dm_specialty_2024 = 3 and licensing_2024 = 1 then clinician_type = 5;
   else if dm_specialty_2024 = 6 and licensing_2024 = 0 then clinician_type = 6;
   else if dm_specialty_2024 = 6 and licensing_2024 = 1 then clinician_type = 7;
   else clinician_type = .;

   /* Flag clinical nurse specialists for exclusion */
   if taxonomy_2024 in ("364SA2200X","364S00000X","364SG0600X","364SF0001X","364SC2300X",
                        "364SH0200X","364SL0600X","364SS0200X","364SP0809X","364SP0808X",
                        "364SP0813X","364SP0807X","364SP0810X","364SP0812X","364SP0811X",
                        "364SN0800X") then do;
      clinician_type = .;
      tax_include_2024 = 0;
   end;

run;

proc sort data=nppes;
   by npi;
run;

/**********************************************************************************/
/* Merge the NPPES taxonomy indicators and active flag to the CY2021 NPI-level    */ 
/* claims-based file.                                                             */
/**********************************************************************************/

data by_npis
     by_npis_export(keep=npi tax_include_2024 taxonomy_2024 active_npi_2024);
   merge &in_by.(in=a) nppes;
   by npi;
   if a;
run;

/* Export CSV of initial NPI-level info for base year */
proc export data=by_npis_export
            outfile="&outnpi1."
            dbms=csv
            replace;
run;

/**********************************************************************************/
/* Read in the FY2023 NPI-level claims-based file creating the 9-category FFS     */
/* setting variable.                                                              */
/**********************************************************************************/

data ry_npis(drop=npi_orig);
   set &in_ry.(rename=(npi = npi_orig));

   length npi $ 10;

   npi = strip(npi_orig);

   if ffs_outpt_only then ffs_setting_check = 1;
   else if ffs_res_only then ffs_setting_check = 2;
   else if ffs_inpt_only then ffs_setting_check = 3;
   else if ffs_outpt_res then ffs_setting_check = 4;
   else if ffs_outpt_inpt then ffs_setting_check = 5;
   else if ffs_res_inpt then ffs_setting_check = 6;
   else if ffs_outpt_res_inpt then ffs_setting_check = 7;
   else if ffs_partd_only then ffs_setting_check = 8;
   else if ffs_notincl_only then ffs_setting_check = 9;

run;

/**********************************************************************************/
/* Merge the NPPES taxonomy indicators and active flag to the FY2023 NPI-level    */ 
/* file above with the 9-category FFS setting variable.                         */
/**********************************************************************************/

data ry_npis
     ry_npis_export(keep=npi tax_include_2024 taxonomy_2024 active_npi_2024);
   merge ry_npis(in=a) nppes;
   by npi;
   if a;
run;

/* Export CSV of initial NPI-level info for most recent year */
proc export data=ry_npis_export
            outfile="&outnpi2."
            dbms=csv
            replace;
run;

/**********************************************************************************/
/* Step 2: Drop NPI without qualifying taxonomies and/or not active               */
/**********************************************************************************/
/**********************************************************************************/
/* Identify CY2021 NPI with relevant taxonomies and who are currently active      */
/**********************************************************************************/

data by_actv_rlvnt_npis
     by_excluded_npis(drop=ffs_ma_2021);
   set by_npis;

   if tax_include_2024 = 1 and active_npi_2024 = 1 then do;
      ffs_ma_2021 = 1;
      output by_actv_rlvnt_npis;
   end;
   else output by_excluded_npis;

run;

/**********************************************************************************/
/* Identify FY2023 NPI with relevant taxonomies and who are currently active      */
/**********************************************************************************/

data ry_actv_rlvnt_npis
     ry_excluded_npis(drop=recent_ffs);
   set ry_npis;

   if tax_include_2024 = 1 and active_npi_2024 = 1 then do;
      recent_ffs = 1;
      output ry_actv_rlvnt_npis;
   end;
   else output ry_excluded_npis;

run;

/**********************************************************************************/
/* Step 3: De-duplicate the set of NPIs across CY2021 and FY2023 by identifying   */
/* NPIs in the CY2021 file that were also in the FY2023 file and dropping their   */
/* CY2021 records so that each NPI is represented only once in the sample frame.  */
/**********************************************************************************/
/**********************************************************************************/
/* Identify NPIs found in CY2021 MA data only vs those who were found in both     */
/* CY 2021 MA and FY 2023 FFS data												  */
/**********************************************************************************/

data by_actv_rlvnt_ma_only_npis;
   merge by_actv_rlvnt_npis(in=a where=(in_maonly)) ry_actv_rlvnt_npis(in=b keep=npi);
   by npi;
   if a;
   exclude_in_rcnt_file = b;
run;

/**********************************************************************************/
/* Create a file of NPIs for exclusion. This includes NPIs who served FFS and MA  */
/* beneficiaries with dementia in CY 2021 or were found in both CY 2021 and FY2023*/
/**********************************************************************************/

data by_excluded_npis_stepb;
   set by_excluded_npis
       by_actv_rlvnt_npis(where=(not in_maonly))
       by_actv_rlvnt_ma_only_npis(where=(exclude_in_rcnt_file));
   by npi;
run;

/**********************************************************************************/
/* Stack the remaining CY2021 NPIs with the FY2023 NPIs. Remaining CY2021 NPIs    */
/* are MA-only and did   not appear in the FY2023 data.                           */
/*                                                                                */
/* Create a four-category variable for whether the NPI was found in CY2021 or     */
/* FY2023 and whether the NPI was a MD/DO or NP/PA.                               */
/**********************************************************************************/

data by_ma_only_and_ry;
   set by_actv_rlvnt_ma_only_npis(where=(not exclude_in_rcnt_file))
       ry_actv_rlvnt_npis;
   by npi;

   if recent_ffs = 1 and licensing_2024 = 0 then npi_type = 1;
   else if recent_ffs = 1 and licensing_2024 in (1,2) then npi_type = 2;
   else if ffs_ma_2021 = 1 and licensing_2024 = 0 then npi_type = 3;
   else if ffs_ma_2021 = 1 and licensing_2024 in (1,2) then npi_type = 4;
   else npi_type = .;

run;

/**********************************************************************************/
/* Step 4: Update the setting variable for NPIs identified as serving             */
/* beneficiaries with dementia in Part D claims only.                             */
/**********************************************************************************/
/**********************************************************************************/
/* Identify NPIs who were only found in Part D data only.                         */
/**********************************************************************************/

data ffs_ptd_only ma_ptd_only;
   set by_ma_only_and_ry(keep=npi ffs_setting_check ma_setting_check);

   if ffs_setting_check = 8 then output ffs_ptd_only;
   if ma_setting_check = 8 then output ma_ptd_only;

run;

/**********************************************************************************/
/* Prep the NPI-level data for FY2023 with setting indicators based on FFS claims */
/* data for all Medicare beneficiaries.                                           */
/**********************************************************************************/

data ry_allpts(drop=npi_orig);
   set &in_ry_allpts.(rename=(npi = npi_orig));

   length npi $ 10;

   npi = strip(npi_orig);

run;

/**********************************************************************************/
/* Merge all-patient setting variable from FY2023 onto FFS NPIs found in Part D   */
/* only.   												    					  */
/**********************************************************************************/

data ffs_ptd_only;
   merge ffs_ptd_only(in=a) ry_allpts(in=b);
   by npi;
   if a;
   if not b then allpts_ffs_setting = 8;
run;

/**********************************************************************************/
/* Merge all-patient setting variable from CY2021 onto MA-only NPIs found in Part */
/* D only*/
/**********************************************************************************/
data ma_ptd_only;
   merge ma_ptd_only(in=a) &in_by_allpts.(in=b);
   by npi;
   if a;
   if not b then allpts_ma_setting = 8;
run;

/**********************************************************************************/
/* Stack FFS and MA Part D-only NPIs with all patient setting variable, and merge */
/* the all-patient setting variable onto the full file of NPIs.                   */
/* Re-set the setting variable to use the all-patient setting indicators for NPIs */ 
/* with setting initially indicating Part D prescriber. Some NPIs have no carrier */
/* or outpatient claims and their setting variable remains Part D prescriber.     */ 
/**********************************************************************************/

data all_ptd_only;
   set ffs_ptd_only ma_ptd_only;
   by npi;
run;

data by_ma_only_and_ry;
   merge by_ma_only_and_ry(in=a) all_ptd_only(drop=ffs_setting_check ma_setting_check);
   by npi;

   if not missing(allpts_ffs_setting) then ffs_setting_check = allpts_ffs_setting;
   if not missing(allpts_ma_setting) then ma_setting_check = allpts_ma_setting;

   if ffs_setting_check = 8 then ffs_ptd_avgclms_pts = ffs_rxcnt_partd/ffs_num_pts;
   if ma_setting_check = 8 then ma_ptd_avgclms_pts = ma_rxcnt_partd/ma_num_pts;

run;

/**********************************************************************************/
/* Step 5: Obtain additional geographic information for NPIs.                     */
/**********************************************************************************/
/**********************************************************************************/
/* Identify whether an NPI practices in outside of the US according to PECOS      */
/**********************************************************************************/

/* Sort PECOS enrollment extract */
proc sort data=&in_enrl.(rename=(state_cd = pecos_state_cd)) out=enrlmnt_extract;
   by npi descending us_non_terr;
run;

/* De-dupe PECOS enrollment extract by NPI giving preference to non-territory records */
data enrlmnt_extract;
   set enrlmnt_extract;
   by npi;
   if first.npi then output;
run;

/* Merge PECOS state code to NPI dataset and create outofusa flag */
data by_ma_only_and_ry(drop=us_non_terr);
   merge by_ma_only_and_ry(in=a) enrlmnt_extract(in=b);
   by npi;
   if a;

   if b then outofusa = (1 - us_non_terr);
   else do;
      if (priority_cntry ne "US") or
         (priority_state in ("AA","AE","AP","AS","GU","MH","MP","PR","VI")) then outofusa = 2;
      else outofusa = 0;
   end;

run;

/**********************************************************************************/
/* Map the NPI's most common practice zip code to its associated county using the */
/* Census crosswalk.                                                              */
/**********************************************************************************/

/* If the modal zip code from the claims processing is missing or invalid   */
/* use the zip code information from the all patients file used above, or   */
/* the zip code from FY2023 NPPES file or the zip code associated with the  */
/* NPPES practice address that was flagged as the priority address          */
data by_ma_only_and_ry;
   set by_ma_only_and_ry;

   length modal_zip $ 5;

   modal_zip = substr(ptb_zip_1,1,5);

   if missing(modal_zip) or modal_zip = "99999" then do;
      if not missing(allpts_zip1) then modal_zip = substr(allpts_zip1,1,5);
      else if not missing(prctc_zip_2023) then modal_zip = substr(prctc_zip_2023,1,5);
      else if not missing(priority_zip) then modal_zip = substr(priority_zip,1,5);
   end;

run;

/* Sort by zip */
proc sort data=by_ma_only_and_ry;
   by modal_zip;
run;

/* Merge with ZIP/ZCTA crosswalk to get ZCTA5 info */
data by_ma_only_and_ry;
   merge by_ma_only_and_ry(in=a) &in_zip_zcta.(rename=(ZIP_CODE = modal_zip
                                                       zcta = ZCTA5));
   by modal_zip;
   if a;
run;

/* Sort by ZCTA5 */
proc sort data=by_ma_only_and_ry;
   by ZCTA5;
run;

/* Merge with ZCTA/County crosswalk to get county (i.e., GEOID) */
data by_ma_only_and_ry(rename=(GEOID = BENE_GEO_CD));
   merge by_ma_only_and_ry(in=a) &in_zcta_cnty.;
   by ZCTA5;
   if a;
run;

/* Sort by renamed GEOID */
proc sort data=by_ma_only_and_ry;
   by BENE_GEO_CD;
run;

/**********************************************************************************/
/* Merge on county-level MA and FFS enrollment from the publicly available        */
/* Medicare Geographic Variations Database (GVDB) from 2021 and 2022 to obtain    */
/* county-level MA participation rates and use the most recent non-missing value. */
/* Calculates the percentage of Medicare beneficiaries enrolled in FFS in each    */
/* county.                                                                        */
/**********************************************************************************/

data by_ma_only_and_ry;
   merge by_ma_only_and_ry(in=a)
         &in_cnty_var.(in=b keep=BENE_GEO_CD GVDB_MA_PRTCPTN_RATE
                            rename=(GVDB_MA_PRTCPTN_RATE = GVDB_MA_PRTCPTN_RATE_2022))
         &in_cnty_var_alt.(in=c keep=BENE_GEO_CD GVDB_MA_PRTCPTN_RATE
                                rename=(GVDB_MA_PRTCPTN_RATE = GVDB_MA_PRTCPTN_RATE_2021));
   by BENE_GEO_CD;
   if a;

   if b then GVDB_MA_PRTCPTN_RATE = GVDB_MA_PRTCPTN_RATE_2022;
   else GVDB_MA_PRTCPTN_RATE = GVDB_MA_PRTCPTN_RATE_2021;

   has_ma_rate = not missing(GVDB_MA_PRTCPTN_RATE);

   if has_ma_rate then gvdb_pct_ffs = 1 - GVDB_MA_PRTCPTN_RATE;
   else gvdb_pct_ffs = .;

   length bene_geo_cd_state $ 2;

   if b or c then bene_geo_cd_state = substr(BENE_GEO_CD,1,2);
   else if not missing(modal_zip) then bene_geo_cd_state = put(zipfips(modal_zip),z2.);
   else bene_geo_cd_state = "";

run;

/* Construct state-level and overall average values for percentage of benes that are FFS */
proc means data=by_ma_only_and_ry noprint;
   class bene_geo_cd_state;
   types () bene_geo_cd_state;
   var gvdb_pct_ffs;
   output out=mean_gvdb_pct_ffs(drop=_type_ _freq_) mean=mean_gvdb_pct_ffs;
run;

/* Separate state-level averages and national average into separate datasets */
data mean_gvdb_pct_ffs_natl(drop=bene_geo_cd_state rename=(mean_gvdb_pct_ffs = mean_gvdb_pct_ffs_natl))
     mean_gvdb_pct_ffs_state(rename=(mean_gvdb_pct_ffs = mean_gvdb_pct_ffs_state));
   set mean_gvdb_pct_ffs;
   if missing(bene_geo_cd_state) then output mean_gvdb_pct_ffs_natl;
   else output mean_gvdb_pct_ffs_state;
run;

/* Sort NPI-level data by state */
proc sort data=by_ma_only_and_ry;
   by bene_geo_cd_state;
run;

/* Merge state-level averages onto NPI-level data */
data by_ma_only_and_ry;
   merge by_ma_only_and_ry mean_gvdb_pct_ffs_state;
   by bene_geo_cd_state;
run;

/* Add national average to each NPI record and use state or national */
/* averages as needed when county-level data was missing             */
data by_ma_only_and_ry(drop=has_ma_rate);
   set by_ma_only_and_ry;
   if _n_ = 1 then set mean_gvdb_pct_ffs_natl;

   if not has_ma_rate then do;
      if not missing(mean_gvdb_pct_ffs_state) then gvdb_pct_ffs = mean_gvdb_pct_ffs_state;
      else gvdb_pct_ffs = mean_gvdb_pct_ffs_natl;
   end;

run;

/* Re-sort by NPI */
proc sort data=by_ma_only_and_ry;
   by npi;
run;

/**********************************************************************************/
/* Step 6: Calculate the total number of beneficiaries with dementia served.      */
/*                                                                                */
/* For NPIs in CY2021 only, the total number of beneficiaries is the actual       */
/* number of FFS and MA beneficiaries with dementia served in CY2021.             */ 
/*                                                                                */
/* For NPIs in both FY2023 and CY2021, we assume the panel size of beneficiaries  */
/* with dementia would be constant over time as well as distribution of           */
/* beneficiaries enrolled in FFS versus MA. As a result, the total panel size of  */
/* Medicare beneficiaries with dementia is estimated as the number of MA          */
/* beneficiaries served in CY2021 plus the number of FFS beneficiaries served in  */
/* FY2023.                                                                        */
/*                                                                                */
/* For NPIs in FY2023 only, the CY 2021 MA panel size is predicted using linear   */
/* regression models based on data from NPIs who were in both CY2021 and FY2023   */
/* files. The final panel size is calculated as the actual total number of FFS    */
/* beneficiaries with dementia in FY2023 plus the predicted number of MA patients */
/* with dementia in CY2021.                                                       */
/**********************************************************************************/

/* Separate NPI-level data into CY2021 MA-only and FY2023 NPIs */
data by_ma_only ry_ffs;
   set by_ma_only_and_ry;

   if npi_type in (1,2) then output ry_ffs;
   else if npi_type in (3,4) then output by_ma_only;

run;

/* Assign patient counts for the CY2021 NPIs based on MA patient counts */
/* from the earlier claims processing                                   */
data by_ma_only;
   set by_ma_only;

   fnl_d_pts = ma_num_pts;
   fnl_d_blckhspnc = ma_num_pts_blck + ma_num_pts_hspnc;
   fnl_d_dualorlis = ma_num_pts_dualorlis;
   fnl_d_pts_dxed = ma_num_pts_dement_dx;

run;

/* For FY2023 NPIs, use CY2021 MA patient counts from earlier              */
/* claims processing to determinhe total patient counts if the NPI         */
/* appeared in CY2021. Otherwise, prep to use subsequent modeling          */
/* steps to assign patient counts                                          */
data ry_ffs;
   merge ry_ffs(in=a) by_npis(in=b keep=npi in_ffsma in_ffsonly in_maonly
                                        ma_num_pts ma_num_pts_blck ma_num_pts_hspnc
                                        ma_num_pts_dualorlis ma_num_pts_dement_dx
                              rename=(in_ffsma = in_ffsma_2021
                                      in_ffsonly = in_ffsonly_2021
                                      in_maonly = in_maonly_2021
                                      ma_num_pts = ma_num_pts_2021
                                      ma_num_pts_blck = ma_num_pts_blck_2021
                                      ma_num_pts_hspnc = ma_num_pts_hspnc_2021
                                      ma_num_pts_dualorlis = ma_num_pts_dualorlis_2021
                                      ma_num_pts_dement_dx = ma_num_pts_dement_dx_2021));
   by npi;
   if a;

   in2021 = b;

   ffs_num_pts_blckhspnc = ffs_num_pts_blck + ffs_num_pts_hspnc;

   if in2021 then do;
      fnl_d_pts = ffs_num_pts + ma_num_pts_2021;
      ma_num_pts_blckhspnc_2021 = ma_num_pts_blck_2021 + ma_num_pts_hspnc_2021;
      fnl_d_blckhspnc = ffs_num_pts_blckhspnc + ma_num_pts_blckhspnc_2021;
      fnl_d_dualorlis = ffs_num_pts_dualorlis + ma_num_pts_dualorlis_2021;
      fnl_d_pts_dxed = ffs_num_pts_dement_dx + ma_num_pts_dement_dx_2021;
   end;
   else do;
      fnl_d_pts = .;
      fnl_d_blckhspnc = .;
      ma_num_pts_blckhspnc_2021 = .;
      fnl_d_dualorlis = .;
      fnl_d_pts_dxed = .;
   end;

run;

/* Execute models for FY2023 NPIs to predict the number of MA patients           */
/* overall, black or Hispanic, and dual-eligible or LIS                          */
proc glm data=ry_ffs;
   title "Model for predicting total base year MA patients";
   class clinician_type ffs_setting_check;
   model ma_num_pts_2021 = ffs_num_pts clinician_type ffs_setting_check gvdb_pct_ffs / solution;
   output out=ma_preds_all p=pred_ma_num_pts;
run;

proc glm data=ry_ffs;
   title "Model for predicting Black/Hispanic base year MA patients";
   class clinician_type ffs_setting_check;
   model ma_num_pts_blckhspnc_2021 = ffs_num_pts_blckhspnc clinician_type ffs_setting_check gvdb_pct_ffs / solution;
   output out=ma_preds_blckhspnc p=pred_ma_num_pts_blckhspnc;
run;

proc glm data=ry_ffs;
   title "Model for predicting DUAL/LIS base year MA patients";
   class clinician_type ffs_setting_check;
   model ma_num_pts_dualorlis_2021 = ffs_num_pts_dualorlis clinician_type ffs_setting_check gvdb_pct_ffs / solution;
   output out=ma_preds_dualorlis p=pred_ma_num_pts_dualorlis;
run;

/* Merge the predicted MA bene counts onto the FY2023 NPI dataset */
data ry_ffs;
   merge ry_ffs
         ma_preds_all(keep=npi pred_ma_num_pts)
         ma_preds_blckhspnc(keep=npi pred_ma_num_pts_blckhspnc)
         ma_preds_dualorlis(keep=npi pred_ma_num_pts_dualorlis);
   by npi;
run;

/* Determine 90th percentile and maximum values for actual MA bene counts */
/* among NPIs that also appeared in the CY2021data                        */
proc means data=ry_ffs noprint;
   var ma_num_pts_2021 ma_num_pts_blckhspnc_2021 ma_num_pts_dualorlis_2021;
   output out=max_ma_pts_vals(drop=_type_ _freq_)
          p90=p90_ma_pts p90_ma_pts_blckhspnc p90_ma_pts_dualorlis
          max=max_ma_pts max_ma_pts_blckhspnc max_ma_pts_dualorlis;
run;

/* Adjust predicted MA bene counts that were outside logical,  */
/* likely value range (e.g. negative or greater than observed  */
/* maximum). Use adjusted predicted values to construct final  */
/* bene counts for FY2023 NPIs that were not found in the      */
/* in CY2021 data.                                             */
data ry_ffs_fnl;  
   set ry_ffs;
   if _n_ = 1 then set max_ma_pts_vals;

   pred_lt_zero = (pred_ma_num_pts < 0);
   pred_bh_lt_zero = (pred_ma_num_pts_blckhspnc < 0);
   pred_dl_lt_zero = (pred_ma_num_pts_dualorlis < 0);

   pred_gt_max = (pred_ma_num_pts > max_ma_pts);
   pred_bh_gt_max = (pred_ma_num_pts_blckhspnc > max_ma_pts_blckhspnc);
   pred_dl_gt_max = (pred_ma_num_pts_dualorlis > max_ma_pts_dualorlis);

   if pred_lt_zero then pred_ma_num_pts = 0;
   if pred_bh_lt_zero then pred_ma_num_pts_blckhspnc = 0;
   if pred_dl_lt_zero then pred_ma_num_pts_dualorlis = 0;

   if pred_gt_max then pred_ma_num_pts = p90_ma_pts;

   if pred_bh_gt_max then do;
      if pred_ma_num_pts_blckhspnc > pred_ma_num_pts then
         pred_ma_num_pts_blckhspnc = min(pred_ma_num_pts,p90_ma_pts_blckhspnc);
      else pred_ma_num_pts_blckhspnc = p90_ma_pts_blckhspnc;
   end;
   else if pred_ma_num_pts_blckhspnc > pred_ma_num_pts then pred_ma_num_pts_blckhspnc = pred_ma_num_pts;

   if pred_dl_gt_max then do;
      if pred_ma_num_pts_dualorlis > pred_ma_num_pts then
         pred_ma_num_pts_dualorlis = min(pred_ma_num_pts,p90_ma_pts_dualorlis);
      else pred_ma_num_pts_dualorlis = p90_ma_pts_dualorlis;
   end;
   else if pred_ma_num_pts_dualorlis > pred_ma_num_pts then pred_ma_num_pts_dualorlis = pred_ma_num_pts;

   pred_ma_num_pts = round(pred_ma_num_pts);
   pred_ma_num_pts_blckhspnc = round(pred_ma_num_pts_blckhspnc);
   pred_ma_num_pts_dualorlis = round(pred_ma_num_pts_dualorlis);

   if not in2021 then do;
      fnl_d_pts = ffs_num_pts + pred_ma_num_pts;
      fnl_d_blckhspnc = ffs_num_pts_blckhspnc + pred_ma_num_pts_blckhspnc;
      fnl_d_dualorlis = ffs_num_pts_dualorlis + pred_ma_num_pts_dualorlis;
   end;

run;

/* Re-stack CY2021 MA-only NPIs and FY2023 NPIs */
data by_ma_only_and_ry;
   set by_ma_only ry_ffs_fnl;
   by npi;
run;

/**********************************************************************************/
/* Step 7: Identify NPIs who served beneficiaries with dementia identified in the */
/* CY2021 MDS only and who were not found in the CY2021 or FY2023 data above.     */
/* Calculate their total panel size by summing the number of CY2021 FFS and MA    */
/* beneficiaries with dementia served and append to the NPI file from Step 6.
/**********************************************************************************/

/* Merge NPPES data onto CY2021 NPIs that were only found in MDS data */
data by_mds_npis;
   merge &in_by_mds.(in=a) nppes(in=b);
   by npi;
   if a;
run;

/* Combine all excluded NPIs from CY2021 and FY2023 */
data all_excluded_npis(keep=npi);
   set by_excluded_npis_stepb ry_excluded_npis;
run;

/* De-dupe excluded NPIs */
proc sort data=all_excluded_npis nodupkey;
   by npi;
run;

/* Merge CY2021 MDS-only NPIs, all excluded NPIs, and the final    */
/* dataset of valid, relevant NPIs from Step 6 and retain only the */
/* MDS-only NPIs that are active and relevant and not in either of */
/* the other two datasets                                          */
data by_mds_missing_npis;
   merge by_mds_npis(in=a) all_excluded_npis(in=b) by_ma_only_and_ry(in=c keep=npi);
   by npi;
   if (a and active_npi_2024 = 1 and tax_include_2024=1 and not (b or c));
run;

/* Create final bene counts for these remaining MDS-only NPIs and */
/* update setting check flags                                     */
data by_mds_missing_npis;
   set by_mds_missing_npis;

   fnl_d_pts = sum(ffs_num_pts,ma_num_pts);
   fnl_d_blckhspnc = sum(ffs_num_pts_blck,ffs_num_pts_hspnc,ma_num_pts_blck,ma_num_pts_hspnc);
   fnl_d_dualorlis = sum(ffs_num_pts_dualorlis,ma_num_pts_dualorlis);
   fnl_d_pts_dxed = sum(ffs_num_pts_dement_dx,ma_num_pts_dement_dx);

   if missing(ffs_setting_check) then do;
      ffs_setting_check = ma_setting_check;
      ma_setting_check = .;
   end;
   else ma_setting_check = .;

   in2021 = .;

run;

/* Merge PECOS enrollment extract info onto remaining MDS-only NPIs */
/* and create outofusa flag                                         */
data by_mds_missing_npis(drop=us_non_terr);
   merge by_mds_missing_npis(in=a) enrlmnt_extract(in=b);
   by npi;
   if a;

   if b then outofusa = (1 - us_non_terr);
   else do;
      if (priority_cntry ne "US") or
         (priority_state in ("AA","AE","AP","AS","GU","MH","MP","PR","VI")) then outofusa = 2;
      else outofusa = 0;
   end;

run;

/**********************************************************************************/
/* Step 8: Combine the MA-only CY2021 and FY2023 NPIs from Step 6 with the        */
/* MDS-only NPIs from Step 7 and create additional setting flags and save         */
/* NPI-level initial sample frame file.                                           */
/**********************************************************************************/

data outfile
     outfile_export(keep=npi);
   set by_ma_only_and_ry(keep=&keepvars.)
       by_mds_missing_npis(in=a keep=&keepvars.);
   by npi;

   mds_only = a;

   /* 4-category mutually exclusive variable for setting 

     NPI was assigned to residential if they provided any services in any residential setting, 
       else to outpatient if they provided any services in outpatient settings, 
         else to Part D prescriber if they were found in Part D claims, 
          else to all other settings. 

   */

   setting_excl_inptonly = .;
   if ffs_setting_check in (2,4,6,7) or ma_setting_check in (2,4,6,7) then setting_excl_inptonly = 1;
   else if ffs_setting_check in (1,5) or ma_setting_check in (1,5) then setting_excl_inptonly = 2;
   else if ffs_setting_check = 8 or ma_setting_check = 8 then setting_excl_inptonly = 3;
   else if ffs_setting_check = 9 or ma_setting_check = 9 then setting_excl_inptonly = 4;

   /* Alternative 5-category mutually exclusive setting variable that flips the order of the residential 
      and outpatient categories and includes a category for providers serving beneficiaries in inpatient settings. 

      NPI was assigned to outpatient if they provided any care in outpatient settings, 
        else to residential if they provided any care in a residential setting, 
          else to inpatient if they provided any care in patient setting, 
            else to Part D prescriber if they were found in Part D claims, 
              else to all other settings. 
   */

   setting_incl_inptonly = .;
   if ffs_setting_check = 1 or ma_setting_check = 1 then setting_incl_inptonly = 1;
   else if ffs_setting_check in (2,4,6,7) or ma_setting_check in (2,4,6,7) then setting_incl_inptonly = 2;
   else if ffs_setting_check in (3,5) or ma_setting_check in (3,5) then setting_incl_inptonly = 3;
   else if ffs_setting_check = 8 or ma_setting_check = 8 then setting_incl_inptonly = 4;
   else if ffs_setting_check = 9 or ma_setting_check = 9 then setting_incl_inptonly = 5;

run;

/* Export CSV of list of NPIs included in sample frame */
proc export data=outfile_export
            outfile="&outnpi3."
            dbms=csv
            replace;
run;

/* Output final SAS dataset of initial sample frame NPI, dropping some unnecessary */
/* variables and masking small cell sizes as needed                                */
data &outfile.;
   set outfile(drop=ffs_setting_check ma_setting_check);
   array vars &keepvars_num.;

   do over vars;
      if 1 <= vars <= 10 then vars = .S;
   end;

run;