/****************************************************************************************************************
   Program: 620_id_npi_tracts.sas

   Purpose: This program finds the census tract of the geocoded NPI addresses.

   Input files: _610_npi_geocoded.sas7bdat
				us_tract_2018_xform.sas7bdat

   Output files: _620_npi_tracts.sas7bdat
****************************************************************************************************************/
options nocenter mprint obs=max;
options validvarname=v7;

libname tract_in "<input directory>" access = readonly;
libname npi_in "<output directory>" access = readonly;
libname out "<output directory>";
%let in_map = tract_in.us_tract_2018_xform;
%let in_npi = npi_in._610_npi_geocoded;
%let outfile = out._620_npi_tracts;

/* Read in census tract mapping file */
data map;
   set &in_map.;
run;

/* Read in NPI data */
data npi;
	set &in_npi.;
run;
proc sql noprint;
	select distinct fips_st_cd into :states separated by " "
	from npi
	where not missing(fips_st_cd);
quit;

/* Find census tract using NPI long/lat */
/* Loop over each state so program runs faster */
%let npi_states_not_in_map = ;
%macro st_loop;

	%do i = 1 %to %sysfunc(countw(&states.));

		/* Limit NPI data to the state */
		%let state = %scan(&states., &i.);
		data npi_&state.;
			set npi (where = (fips_st_cd = "&state."));
		run;

		/* Limit map data to the state */
		data map_&state.;
			set map (where = (st_fips = "&state."));
		run;
		proc sql noprint;
			select count(*) into :map_state_obs
			from map_&state.;
		quit;

		/* Determine census tract */
		%if &map_state_obs. > 0 %then %do; /* if FIPs code from NPI data exists in the tract data */
			proc ginside data = npi_&state. map = map_&state. out = npi_tracts_&state. (keep = npi geoid segment _onborder_)
				includeborder insideonly;
				id geoid segment;
			run;
		%end;
		%else %do;
			%let npi_states_not_in_map = &npi_states_not_in_map. &state.;
		%end;

	%end;

%mend st_loop;
%st_loop;

/* Stack by state NPI data with census tracts */
data npi_all_tracts_initial;
   set npi_tracts_:;
   rename _onborder_ = on_border;
run;

/* Merge NPI data with census tracts to original NPI data */
proc sql;
	create table npi_all_tracts_initial2 as
	select a.*,
		   b.segment,
		   b.geoid,
		   b.on_border
	from npi as a
	left join
	npi_all_tracts_initial as b
	on a.npi = b.npi;
quit;

/* Find NPIs without a census tract that have long/lat (just no FIPs) and try to find a census tract using the entire map data */
data npi_all_tracts_initial3 npi_reprocess;
	set npi_all_tracts_initial2;
	if missing(geoid) and not missing(x) then output npi_reprocess;
	else output npi_all_tracts_initial3;
run;
proc ginside data = npi_reprocess map = map out = npi_tracts_reprocess (keep = npi geoid segment _onborder_)
	includeborder insideonly;
	id geoid segment;
run;
data npi_tracts_reprocess;
	set npi_tracts_reprocess;
   	rename _onborder_ = on_border;
run;
data npi_all_tracts_initial4;
   set npi_all_tracts_initial3 npi_tracts_reprocess;
run;

/* Merge again with initial NPI data */
proc sql;
	create table npi_all_tracts_final as
	select a.*,
		   b.segment,
		   b.geoid,
		   b.on_border
	from npi as a
	left join
	npi_all_tracts_initial4 as b
	on a.npi = b.npi;
quit;

/* Save final dataset */
data &outfile.;
   set npi_all_tracts_final (keep = npi st_cd geoid on_border);
run;

/* Clean up environment */
proc datasets lib = work kill nolist;
run; quit;
