/****************************************************************************************************************
   Program: 		650_id_npi_tracts.sas

   Purpose: 		Use census tract boundary file to identify census tracts via lat/long for NPIs' NPPES addresses.

   Input files: 	us_tract_2018_xform.sas7bdat
					State_FIPS_Lookup.xlsx
                	_640_npi_geocoded.sas7bdat

   Output files: 	_650_npi_tracts.sas7bdat
****************************************************************************************************************/
/* Setup environment */
options nocenter mprint validvarname = v7 obs = max;

%let inpath = <input directory>;
%let outpath = <output directory>;
libname in1 "&inpath." access=readonly;
libname in2 "&outpath." access=readonly;
libname out "&outpath.";
%let in_map = in1.us_tract_2018_xform;
%let in_npi = in2._640_npi_geocoded;
%let in_lkup = &inpath.\State_FIPS_Lookup.xlsx;
%let outfile = out._650_npi_tracts;

/* Read in census tract mapping file */
data map;
   set &in_map.;
run;

/* Read in state fips lookup */
proc import datafile = "&in_lkup." out = st_fips_lkup_initial
	dbms = xlsx replace;
	sheet = "State_FIPS_Lookup";
run;
data st_fips_lkup;
	set st_fips_lkup_initial (drop = state_short rename = (state = state_short)) 
		st_fips_lkup_initial (drop = state);
	state_short = upcase(state_short);
run;
proc sort data = st_fips_lkup;
	by state_short;
run;

/* Merge on state FIPs code to NPI data */
data npi_initial;
	set &in_npi.;
	length state_short $20.;
 	state_short = upcase(priority_state);
run;
proc sort data = npi_initial;
	by state_short;
run;
data npi;
	merge npi_initial (in = a) st_fips_lkup (in = b);
	by state_short;
	if a;
	in_npi = a;
	in_st_fips_lkup = b;
run;
proc sql noprint;
	select distinct fips into :states separated by " "
	from npi
	where not missing(fips);
quit;
%put States To Loop Over: &states.;

/* Find census tract using NPI long/lat */
/* Loop over each state so program runs faster */
%let npi_states_not_in_map = ;
%macro st_loop;

	%do i = 1 %to %sysfunc(countw(&states.));

		/* Limit NPI data to the state */
		%let state = %scan(&states., &i.);
		data npi_&state.;
			set npi (where = (fips = "&state."));
		run;

		/* Limit map data to the state */
		data map_&state.;
			set map (where = (st_fips = "&state."));
		run;
		proc sql noprint;
			select count(*) into :map_state_obs
			from map_&state.;
		quit;

		/* Determine census tract */
		%if &map_state_obs. > 0 %then %do; /* if FIPs code from NPI data exists in the tract data */
			proc ginside data = npi_&state. map = map_&state. out = npi_tracts_&state. (keep = npi geoid segment _onborder_)
				includeborder insideonly;
				id geoid segment;
			run;
		%end;
		%else %do;
			%let npi_states_not_in_map = &npi_states_not_in_map. &state.;
		%end;

	%end;

%mend st_loop;
%st_loop;
%put States Not in Map: &npi_states_not_in_map.;

/* Stack by state NPI data with census tracts */
data npi_all_tracts_initial;
   set npi_tracts_:;
   rename _onborder_ = on_border;
run;

/* Merge NPI data with census tracts to original NPI data */
proc sql;
	create table npi_all_tracts_initial2 as
	select a.*,
		   b.segment,
		   b.geoid,
		   b.on_border,
		   case
		   		when missing(a.npi) then "Only in NPI Data with Census Tracts"
		   		when missing(b.npi) then "Only in Original NPI Data"
				else "NPI Merged"
		   end as npi_tract_merge
	from npi as a
	left join
	npi_all_tracts_initial as b
	on a.npi = b.npi;
quit;

/* Find NPIs without a census tract that have long/lat (just no FIPs) and try to find a census tract using the entire map data */
data npi_all_tracts_initial3 npi_reprocess;
	set npi_all_tracts_initial2;
	drop npi_tract_merge;
	if missing(geoid) and not missing(x) then output npi_reprocess;
	else output npi_all_tracts_initial3;
run;
proc ginside data = npi_reprocess map = map out = npi_tracts_reprocess (keep = npi geoid segment _onborder_)
	includeborder insideonly;
	id geoid segment;
run;
data npi_tracts_reprocess;
	set npi_tracts_reprocess;
	rename _onborder_ = on_border;
run;
data npi_all_tracts_initial4;
   set npi_all_tracts_initial3 npi_tracts_reprocess;
run;

/* Merge again with initial NPI data */
proc sql;
	create table npi_all_tracts_final as
	select a.*,
		   b.segment,
		   b.geoid,
		   b.on_border,
		   case
		   		when missing(a.npi) then "Only in NPI Data with Census Tracts"
		   		when missing(b.npi) then "Only in Original NPI Data"
				else "NPI Merged"
		   end as npi_tract_merge
	from npi as a
	left join
	npi_all_tracts_initial4 as b
	on a.npi = b.npi;
quit;

/* Save and QA final dataset */
data &outfile.;
   set npi_all_tracts_final (keep = npi geoid on_border);
run;

/* Clean up environment */
proc datasets lib = work kill nolist;
run; quit;