/****************************************************************************************************************
   Program: 700_cc_sample_frame.sas

   Purpose: Create the final Community Clinician sample frame file by merging in IQVIA and NPPES RUCA/rural status 
			and excluding NPIs who work outside of the US or who work in other settings only or inpatient settings only.
	
   Input files: NPPES Priority Practice Location Geocoded to assign RUCA / Rural status
                 _660_npi_rural.sas7bdat
                IQVIA Practice Location Geocoded to assign RUCA / Rural status
                 _630_npi_rural.sas7bdat
                Y1 CC Sample Frame
                 ndws_cc_sample_frame.sas7bdat

   Output files: CC Sample Frame with Rural
                   ndws_ccsf_{date}.sas7bdat
****************************************************************************************************************/
options nocenter ls=250 ps=90 nofmterr formdlim="-" FORMCHAR = '|----|+|---+=|-/<>*'
        mprint mlogic symbolgen noquotelenmax validvarname=v7 msglevel = i ;

/*********************************************************************************************************
 Libnames
*********************************************************************************************************/

libname cc "<input library>" access=readonly;
libname nppes "<nppes library>" access=readonly;
libname iqvia "<iqvia library>" access=readonly;

libname out "<output library>";

/*********************************************************************************************************
 Input files
*********************************************************************************************************/

%let ccsf = cc.ndws_cc_sample_frame;
%let nppes = nppes._660_npi_rural;
%let iqvia = iqvia._630_npi_rural;

/*********************************************************************************************************
 Output files
*********************************************************************************************************/

%let outframe = out.ndws_ccsf_&sysdate.;

/*********************************************************************************************************
 Formats
*********************************************************************************************************/

proc format;
  value specialty
  1 = "1=PCP: MD or DO"
  2 = "2=PCP: NP"
  3 = "3=PCP: PA"
  4 = "4=PSYCH: MD or DO"
  5 = "5=PSYCH: NP"
  6 = "6=NEURO: MD or DO"
  7 = "7=NEURO: NP";

  value rural
  0 = "0=Urban"
  1 = "1-Rural";

  value setting
  1 = "1:Any Residential"
  2 = "2:Outpatient, non-residential"
  3 = "3:Part D prescriber only"
  4 = "4:NES and not exclusively inpatient";
  
  value missing
  . = "Missing"
  99 = "99"
  other = "Not Missing";
run;

/*********************************************************************************************************
 Read in community clinician sample frame from VRDC
*********************************************************************************************************/

proc sort data=&ccsf. out=ccsf;
  by npi;
run;

/*********************************************************************************************************
 Merge rural indicators from NPPES and IQVIA onto the sample frame and compare rural status
*********************************************************************************************************/

data nppes;
  set &nppes. (rename=(npi = npi_num rural = nppes_rural primary_ruca_cd = nppes_ruca));
  length npi $10;
  npi = put(npi_num,10.);
  drop npi_num;
run;

proc sort data=nppes;
  by npi;
run;

data iqvia;
  set &iqvia. (rename=(npi = npi_num rural = iqvia_rural primary_ruca_cd = iqvia_ruca
                       st_cd = iqvia_state));
  length npi $10;
  npi = put(npi_num,10.);
  drop npi_num;

  /* Flag IQVIA states that indicate out of US or in US Territory */
  if missing(iqvia_state) then iqvia_outofus = .;
  else if iqvia_state in ('AE' 'AP' 'AS' 'GU' 'MP' 'PR' 'VI') then iqvia_outofus = 1;
  else iqvia_outofus = 0;
run;

proc sort data=iqvia;
  by npi;
run;

data ccsf_rural;
  merge ccsf (in=cc)
        nppes (in=nppes drop=tract_fips_cd)
        iqvia (in=iqvia drop=tract_fips_cd);
  by npi;
  /* Keep all NPI in the CC sample frame */
  if cc;
  InCCSF = cc;
  InNPPES = nppes;
  InIQVIA = iqvia;

  /* Confirm the merge does not introduce duplicate NPIs */
  if first.npi and not last.npi then abort;
run;

/*********************************************************************************************************
 Assign rural status
*********************************************************************************************************/

data ccsf_rural_staus;
  set ccsf_rural;

  /* Backfill IQVIA RUCA / rural if missing with NPPES data */

  iqvia_rural_orig = iqvia_rural;
  if missing(iqvia_rural) and ~missing(nppes_rural) then iqvia_rural = nppes_rural;

  iqvia_ruca_orig = iqvia_ruca;
  if iqvia_ruca in (.,99) and nppes_ruca ~in (.,99) then iqvia_ruca = nppes_ruca;

run;

/*********************************************************************************************************
 Determine whether NPI works outside of the US
*********************************************************************************************************/

data ccsf_outofus;
  set ccsf_rural_staus (in=cc rename=(outofusa = outofusa_pecos_nppes));

  /* Flag Out of US for excluding NPI */
  if iqvia_ruca in (.,99) and outofusa_pecos_nppes in (1,2)
	  then outofusa = 1;
  else if iqvia_outofus = 1 then outofusa = 2;
	  else outofusa = 0;
run;

/*********************************************************************************************************
 Exclude NPIs working outside of the US and those working in other settings only or inpatient settings only
*********************************************************************************************************/

data cc_sample_frame (rename=(setting_excl_inptonly = final_setting));
  set ccsf_outofus;
  if outofusa = 0 and setting_excl_inptonly in (1,2,3);

  /* Set missing rural to 0 and flag those we reset */
  iqvia_rural_missing = 0;
  if missing(iqvia_rural) then do;
    iqvia_rural_missing = 1;
	iqvia_rural = 0;
  end;
run;

/*********************************************************************************************************
 For cell sizes smaller than 11, set to 5 
*********************************************************************************************************/

data cc_sf;
  set cc_sample_frame;
  if missing(fnl_d_pts) then do; fnl_d_pts = 5; end; 
  if missing(fnl_d_blckhspnc) then do; fnl_d_blckhspnc = 5; end;
  if missing(fnl_d_dualorlis) then do; fnl_d_dualorlis = 5; end;
  if fnl_d_pts_dxed = .S then do; fnl_d_pts_dxed = 5; end;
run;

/*********************************************************************************************************
 Save CC sample frame file
*********************************************************************************************************/

data &outframe.;
  retain npi               
         PROV_NM_PRE_2024  
         PROV_FN_2024      
         PROV_LN_2024      
         PROV_NM_SUFF_2024 
         taxonomy_2024     
         clinician_type    
         iqvia_rural             
         iqvia_rural_missing   
         final_setting 
		 fnl_d_pts
		 fnl_d_blckhspnc
		 fnl_d_dualorlis
		 fnl_d_pts_dxed
         PRIORITY_ADDRESS_1                             
         PRIORITY_ADDRESS_2                             
         PRIORITY_CITY                                  
         PRIORITY_CNTRY                                 
         PRIORITY_STATE                                 
         PRIORITY_ZIP                                   
		 ;

  set cc_sf;

  keep npi               
       PROV_NM_PRE_2024  
       PROV_FN_2024      
       PROV_LN_2024      
       PROV_NM_SUFF_2024 
       taxonomy_2024     
       clinician_type    
       iqvia_rural             
       iqvia_rural_missing   
       final_setting 
	   fnl_d_pts
	   fnl_d_blckhspnc
	   fnl_d_dualorlis
	   fnl_d_pts_dxed
       PRIORITY_ADDRESS_1                             
       PRIORITY_ADDRESS_2                             
       PRIORITY_CITY                                  
       PRIORITY_CNTRY                                 
       PRIORITY_STATE                                 
       PRIORITY_ZIP                                   
	   ;

   label npi                 = "Community Clinician NPI"
         PROV_NM_PRE_2024    = "2024 Provider Prefix"     
         PROV_FN_2024        = "2024 Provider First Name"    
         PROV_LN_2024        = "2024 Provider Last Name (Legal)"    
         PROV_NM_SUFF_2024   = "2024 Provider Suffix"   
         taxonomy_2024       = "Primary Taxonomy Code"    
         clinician_type      = "Clinician Type 1=PCP:MD DO, 2=PCP:NP, 3=PCP:PA, 4=PSYCH:MD DO, 5=PSYCH:NP, 6=NEURO:MD DO"    
         iqvia_rural         = "Rural flag based on IQVIA practice location, backfilled with NPPES when IQVIA rural status missing"    
         iqvia_rural_missing = "1=Rural flag missing and was set to 0 (not rural)" 
         final_setting       = "Setting: 1=any residential, 2=outpatient (non-residential), 2-Part D prescriber only"    
         fnl_d_pts           = "Total number of dementia patients (FY2023 FFS+CY2021 MA (predicted CY2021 MA for clinicians not in practice in 2021)"  
         fnl_d_blckhspnc     = "Total number of Black and/or Hispanic dementia patients (FY2023 FFS+CY2021 MA (predicted CY2021 MA for clinicians not in practice in 2021)"    
         fnl_d_dualorlis     = "Total number of Dually Eligible or LIS dementia patients (FY2023 FFS+CY2021 MA (predicted CY2021 MA for clinicians not in practice in 2021)"    
         fnl_d_pts_dxed      = "Total number of dementia patients clinician diagnosed with dementia (FY2023 FFS+CY2021 MA patients that the NPI diagnosed with dementia)"
         PRIORITY_ADDRESS_1  = "NPPES Priority Address 1"                          
         PRIORITY_ADDRESS_2  = "NPPES Priority Address 2"                           
         PRIORITY_CITY       = "NPPES Priority City"                          
         PRIORITY_CNTRY      = "NPPES Priority Country"                          
         PRIORITY_STATE      = "NPPES Priority State"                          
         PRIORITY_ZIP        = "NPPES Priority Zip"                          
  ;

  format final_setting setting. clinician_type specialty. iqvia_rural rural.;
run;